package animation;

import gameobjects.AbstractGameObject;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import com.golden.gamedev.object.Timer;

/**
 * This class can be used to schedule animation sequences
 * 
 * @author Frank Bruns
 */
public class AnimationScheduler implements Serializable
{
	private AbstractGameObject go = null;
	private LinkedList<Animation> queue = null;
	private boolean running = false;
	
	private long timerTime = 0;

	public AnimationScheduler(AbstractGameObject go)
	{
		this.queue = new LinkedList<Animation>();
		this.go = go;			
	}

	/**
	 * Adds an animation to the queue
	 * 
	 * @param name
	 *            animation reference name
	 * @param dur
	 *            animation duration in miliseconds
	 */
	public synchronized void add(String name, long dur)
	{
		this.queue.add(new Animation(this.go.getAnimPool().getAnimSequence(
				name), name, dur));
	}
	
	/**
	 * Adds an animation with infinite duration to the queue. Such animation 
	 * can only be eliminated from the queue by clearing the whole queue with
	 * the clear() method 
	 * @param name animation refrence name
	 */
	public synchronized void add(String name)
	{
		this.queue.add(new Animation(this.go.getAnimPool().getAnimSequence(
				name), name));
	}

	/**
	 * Returns the reference name of the currently active animation on schedule
	 * 
	 * @return constant of the active animation
	 */
	public String getCurrAnimName()
	{
		return this.queue.getFirst().getName();
	}
	
	/**
	 * Returns the name of the animation that follows on the
	 * current animation in the animation scheduler queue. If there is
	 * none the method return <code>null</code>
	 * @return name of the following (second) animation in the queue
	 */
	public String getNextAnimName()
	{
		if(this.queue.size() > 1)
		{
			return this.queue.get(1).getName();
		}
		else return null;
	}
	
	/**
	 * Removes the first element (current animation) from the
	 * animation queue, if there is one
	 *
	 */
	public void removeFirst()
	{
		if(this.queue.getFirst() != null) this.queue.removeFirst();
	}

	/**
	 * Starts the processing of the animation queue from the beginning
	 */
	public void start()
	{
		this.running = true;
		if (this.queue.size() > 0)
		{
			this.timerTime = this.queue.getFirst().getDuration();
		}
		else this.next();
	}

	/**
	 * Stops the processing of the animation queue
	 */
	public void pause()
	{
		this.running = false;
	}

	/**
	 * Resumes the schedule
	 */
	public void resume()
	{
		this.running = true;
	}

	/**
	 * Deletes the animations of the processing queue.
	 */
	public synchronized void clear()
	{
		this.queue.clear();	
		this.timerTime = 0;
	}

	/**
	 * Sets the animation scheduler to the next animation. Only is effective if
	 * there actually _is_ another animation on schedule. Animations that
	 * were set to run infintely will not be skipped for the next
	 * animation.
	 */
	public synchronized void next()
	{		
		// if there is still an element left, use it for animation,
		// else set the standard animations
		if (this.queue.size() > 0)
		{
			if(this.queue.getFirst().isRunning() == false)
			{
				this.queue.getFirst().setRunning(true);
				this.go.setImages(this.queue.getFirst().getImages());
				this.timerTime = this.queue.getFirst().getDuration();
			}
			else
			{
				if(!this.queue.getFirst().isInfinite())this.queue.removeFirst();
			}
		}
		else
		{
			this.add("standard");
			this.queue.getFirst().setRunning(true);
			this.go.setImages(this.queue.getFirst().getImages());			
			this.timerTime = this.queue.getFirst().getDuration();
		}
	}
	
	/**
	 * Sets the animation scheduler to the next animation. Only is effective if
	 * there actually _is_ another animation on schedule. Compared to the
	 * simple next() method this method is able to even skip an
	 * animation that was set to run inifitely.
	 * animation.
	 */
	public synchronized void forceNext()
	{		
		// if there is still an element left, use it for animation,
		// else set the standard animations
		if (this.queue.size() > 0)
		{
			if(this.queue.getFirst().isRunning() == false)
			{
				this.queue.getFirst().setRunning(true);
				this.go.setImages(this.queue.getFirst().getImages());
				this.timerTime = this.queue.getFirst().getDuration();
			}
			else
			{
				this.queue.removeFirst();
			}
		}
		else
		{
			this.add("standard");
			this.queue.getFirst().setRunning(true);
			this.go.setImages(this.queue.getFirst().getImages());			
			this.timerTime = this.queue.getFirst().getDuration();
		}
	}

	/**
	 * Returns the queue of the animation scheduler
	 * 
	 * @return animation scheduler queue
	 */
	public LinkedList<Animation> getQueue()
	{
		return this.queue;
	}

	/**
	 * Updates the animation schedule
	 * 
	 * @param elapsedTime
	 *            time between two frames
	 */
	public synchronized void update(long elapsedTime)
	{
		if (this.running)
		{
			this.timerTime -= elapsedTime;
					
			if(this.timerTime <= 0)
			{		
				this.next();												
			}
		}
	}
}
