package gameaction;

import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.Player;

import java.awt.Graphics2D;
import java.io.Serializable;
import java.util.LinkedList;
import state.GameState;

/**
 * This class manages a queue of game actions
 * @author Frank Bruns
 *
 */
public class GameActionScheduler implements Serializable
{
	private LinkedList<AbstractGameAction> gameActions = null;
	
	private boolean pause = false;
	private AbstractCharacterSprite character = null;
	
	/**
	 * Creates a new game action scheduler with the specified
	 * character as the owner. Just provide a this reference for the
	 * owner character when initializing the manager for an
	 * AbstractCharacterSprite
	 * @param c character this game action manager got instanciated for
	 */
	public GameActionScheduler(AbstractCharacterSprite c)
	{
		this.character = c;
		this.gameActions = new LinkedList<AbstractGameAction>();
	}
	
	/**
	 * Puts the game action scheduler to a paused state. Game actions will not
	 * be processed then, but can be added though.
	 *
	 */
	public void pause()
	{
		this.pause = true;
	}
	
	/**
	 * Resumes the game action manager, if it was in an paused state
	 *
	 */
	public void resume()
	{
		if(this.pause == true) this.pause = false;
	}
	
	/**
	 * Adds a game action to the queue of game actions.
	 * @param action game action to add to the game action queue
	 */
	public void add(AbstractGameAction action)
	{
		this.gameActions.add(action);
	}
	
	/**
	 * Removes every game action from the queue of game actions
	 *
	 */
	public void clear()
	{
		this.gameActions.clear();
	}
	
	/**
	 * Removes first element in the game action queue
	 *
	 */
	public void removeFirst()
	{
		this.gameActions.removeFirst();
	}
	
	/**
	 * Returns first element of the game action queue
	 * @return first element of the game action queue
	 */
	public AbstractGameAction getFirst()
	{
		return this.gameActions.getFirst();
	}
	
	/**
	 * Returns the list of game actions
	 * @return list of game actions
	 */
	public LinkedList<AbstractGameAction> getGameActions()
	{
		return this.gameActions;
	}
	
	/**
	 * This methods determines whether there a game actions the queue
	 * that are other than walk game actions
	 * @return Are there game actions in the queue that are not walk actions
	 */
	public boolean containsActionsOtherThanWalk()
	{
		for(AbstractGameAction action : this.gameActions)
		{
			if((action instanceof Walk) == false) return true;
		}
		return false;
	}
	
	/**
	 * Uodates the queue of game actions
	 * @param elapsedTime time between two frames
	 */
	public void update(long elapsedTime)
	{
		//System.out.println(this.character.getName()+" actions: "+this.gameActions.size());
		if(this.pause == false)
		if(this.gameActions.isEmpty() == false)
		{			
			
			AbstractGameAction action = this.gameActions.getFirst();					
			
			// if a game action for another character is in the 
			// queue of this character then call that character's
			// game action scheduler update() method 
			if(action.getCharacter().equals(this.character) == false)
			{
				action.getCharacter().getGameActionScheduler().add(action);
				this.gameActions.removeFirst();
			}
			else action.update(elapsedTime);
			
			
			//this.gameActions.getFirst().update(elapsedTime);
		}
	}
	
	public void render(Graphics2D g)
	{
		if(this.pause == false)
		if(this.gameActions.isEmpty() == false)
		{
			AbstractGameAction action = this.gameActions.getFirst();
			
			// handle rendering talk game action
			if(action instanceof Talk)
			{
				Talk talk = (Talk)action;
				talk.render(g);
			}
			
			if(action instanceof CutSceneStart)
			{
				CutSceneStart csStart = (CutSceneStart)action;
				csStart.render(g);
			}
			
			if(action instanceof CutSceneStop)
			{
				CutSceneStop csStop = (CutSceneStop)action;
				csStop.render(g);
			}
		}		
	}
}
