package gameobjects;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.io.Serializable;

import state.GameState;

import location.GameLocation;
import location.Hotspot;

import animation.AnimationPool;
import animation.AnimationScheduler;
import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;
import com.golden.gamedev.object.AnimatedSprite;
import com.golden.gamedev.object.font.SystemFont;

/**
 * This class is the basic class of all game objects. It contains all the
 * methods and variables that all game objects have in common
 * @author Frank Bruns
 *
 */
public abstract class AbstractGameObject extends AnimatedSprite implements Serializable, Comparable<AbstractGameObject>
{	
	protected BaseLoader bsLoader = null;	
	protected BaseInput bsInput = null;
	protected BaseAudio bsSound = null;
	
	private GameLocation location = null;
	protected Hotspot sceneHotspot = null;
	private String name = "no name yet";
	
	/**
	 * This is the name that the game object will be referenced by in 
	 * the beanshell scripts
	 */
	private String bshName = "no beanshell name yet";
	/**
	 * The description for the item. Please change it in your subclass.
	 */

	private String desc = "no description yet";
	
	protected SystemFont screenWriter = null;
	protected AnimationPool animPool = null;
	protected AnimationScheduler animScheduler = null;
	
	private boolean isPicked = false;
	
	//private int width = -1;
	//private int height = -1;
	//private double previousY = -1;
	
	/**
	 * Creates a new game object with the specified name at the specified coordinates
	 * @param name name of the game object
	 * @param bshName name that will be used for this object in beanshell
	 * scripts
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param bsLoader
	 * @param bsInput
	 * @param bsSound
	 */
	public AbstractGameObject(String name, String bshName, double x, double y, BaseLoader bsLoader, BaseInput bsInput, BaseAudio bsSound)
	{
		this.name = name;
		this.bshName = bshName;
		
		this.bsLoader = bsLoader;
		this.bsInput = bsInput;
		this.bsSound = bsSound;		
		
		this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,15),Color.WHITE);
		this.animPool = new AnimationPool();
		this.animScheduler = new AnimationScheduler(this);
		
		this.setAnimate(true);
		this.setLoopAnim(true);
		this.getAnimationTimer().setDelay(150);
		this.setLocation(x,y);				
		
		this.sceneHotspot = new Hotspot(name,this.getX(),this.getY()-this.height,this.getWidth(),this.getHeight(),bsInput);
	}
	
	public int compareTo(AbstractGameObject o)
	{		
		if(this.getY() > o.getY()) return 1;
		if(this.getY() < o.getY()) return -1;
		return 0; // for this.getY() == o.getY()
	}
	
	/**
	 * Returns the y coordinate of the game object as the lower
	 * corner of the game object
	 */
	/*
	@Override
	public double getY()
	{
		return super.getY()-this.getHeight();
	}
	*/
	
	/**
	 * Returns the middle x coordinate of the character
	 * @return middle x coordinate of the character
	 */
	public double getMidX()
	{
		return (this.getX()+(double)this.width)/2.0;
	}
	 
	/**
	 * Returns the middle y coordinate of the character
	 * @return middle y coordinate of the character
	 */
	public double getMidY()
	{
		return (super.getY()+(double)this.height)/2.0;
	}
	
	/**
	 * Returns the base loader object
	 * @return base loader object
	 */
	public BaseLoader getBsLoader()
	{
		return this.bsLoader;
	}
	
	/**
	 * Returns the pool of this game object's animation sequences
	 * @return pool of animation sequences
	 */
	public AnimationPool getAnimPool()
	{
		return this.animPool;
	}
	
	/**
	 * Sets the item to picked status
	 *
	 */
	public void setToPicked()
	{
		this.isPicked = true;
	}
	
	/**
	 * Returns whethter this item has been picked by the player
	 * @return has been picked by the player? true=yes, false=no
	 */
	public boolean isPicked()
	{
		return this.isPicked;
	}
	
	/**
	 * Set the game location of the game object
	 * @param loc game location of the game object
	 */
	public synchronized void setGameLocation(GameLocation loc)
	{
		this.location = loc;
	}
	
	/**
	 * Returns the game location of the game object 
	 * @return game location of the game object
	 */
	public synchronized GameLocation getGameLocation()
	{
		return this.location;
	}
	
	/**
	 * Returns the name of the game object
	 * @return name of the game object
	 */
	public String getName()
	{
		return this.name;
	}	
	
	/**
	 * Sets the name of the character
	 * @param name new name of the character
	 */
	public void setName(String name)
	{
		this.name = name;
	}
	
	/**
	 * Returns the beanshell name of this game object. This is the name
	 * that can be used in beanshell scripts to refer to this game object.
	 * @return beanshell name of the game object
	 */
	public String getBshName()
	{
		return this.bshName;
	}
	
	/**
	 * Sets the description for the game object.
	 * @param desc description for the item
	 * @deprecated The system has no use for the method anymore
	 */
	public void setDescription(String desc)
	{
		this.desc = desc;
	}
	
	/**
	 * Returns the description of the game object
	 * @return item's description
	 * @deprecated The system has no use for the method anymore
	 */
	public String getDescription()
	{
		return this.desc;
	}
	
	/**
	 * Returns the hotspot of the item
	 * @return hotspot of the item
	 */
	public Hotspot getHotspot()
	{
		return this.sceneHotspot;
	}
	
	/**
	 * Returns the animation scheduler
	 * @return animation scheduler
	 */
	public AnimationScheduler getAnimScheduler()
	{
		return this.animScheduler;
	}

	@Override
	public void update(long elapsedTime)
	{		
		super.update(elapsedTime);		
		this.animScheduler.update(elapsedTime);
		this.sceneHotspot.update(this.getX(), this.getY()-this.height, this.width, this.height);
		
//		// init the width and height values with the original ones
//		if(this.width == -1) this.width = super.width;
//		if(this.height == -1) this.height = super.height;
//		if(this.previousY == -1) this.previousY = this.getY();
//								
//		
//		if(this.getY() > this.previousY)
//		{
//			float scaleFactor = this.getGameLocation().getScaling();
//			this.width = (int)((float)super.width+(super.width+scaleFactor*100));
//			this.height = (int)((float)super.height+(super.height+scaleFactor*100));	
//		}
//		else
//		{
//			float scaleFactor = this.getGameLocation().getScaling();
//			this.width = (int)((float)super.width-(super.width+scaleFactor*100));
//			this.height = (int)((float)super.height-(super.height+scaleFactor*100));	
//		}
//		
//		this.previousY = this.getY();		
	}		
	
	
	
	@Override
	public void render(Graphics2D g)
	{
//		
//		if(this instanceof Item)
//		{
//			super.render(g, (int)this.getX(), (int)this.getY()-this.height);
//		}
//		else // apply scaling
//		{							
//			g.drawImage(
//					this.getImage().
//					getScaledInstance(this.width, 
//							this.height, 
//							Image.SCALE_FAST),
//							(int)this.getX(),
//							(int)this.getY()-this.height,
//							null);							
//			
//			/*
//			// init the prevevious y value
//			if(this.width == 0 && this.height == 0) this.previousY = this.getY();
//			
//			if((this.width > 0 && this.height > 0) 
//					&&(this.previousY != this.getY()))
//			{							
//				float scaleFactor = 767f /(100-this.getGameLocation().getScaling());
//				
//				int scaledWidth = this.getWidth();
//				int scaledHeight = this.getHeight();
//				if(this.previousY > this.getY()) scaledWidth = this.width-1;
//				if(this.previousY < this.getY()) scaledHeight = this.width+1;
//				
//				// scale the animation images
//				BufferedImage[] scaledImages = this.getImages();
//				for(int i=0; i<scaledImages.length; i++)
//				{
//					scaledImages[i] = ImageUtil.resize(scaledImages[i],
//							scaledWidth,
//							scaledHeight);
//				}
//				
//				// save the current frame to be able to restore it
//				// after the resized images were set to the animation
//				int animFrame = this.getFrame();
//				this.setImages(scaledImages);
//				this.setFrame(animFrame);
//				
//				this.previousY = this.getY();
//			}
//			
//			super.render(g, (int)this.getX(), (int)this.getY()-this.height);
//			*/
//		}
		
		super.render(g, (int)this.getX(), (int)this.getY()-this.height);
		
		/*
		if(GameState.equals(GameState.TRANSITION_TO_CUTSCENE)
				|| GameState.equals(GameState.TRANSITION_FROM_CUTSCENE))
		{
			this.getGameLocation().disableRendering();
			g.fillRect(0, 0, 1024, 768);
		}
		else if(this.getGameLocation() != null
				&& this.getGameLocation().isStoppedRendering() == true)
		{
			this.getGameLocation().enableRendering();
		}
		*/
	}

	/*
	@Override
	public int getHeight()
	{	
		return this.height;
	}

	@Override
	public int getWidth()
	{
		return this.width;
	}
	*/

	/**
	 * Shows the name of the game object below or above it.
	 * @param g
	 * @param below show name below game object? -->true=yes, false=no
	 */
	/*
	public void showName(Graphics2D g, boolean below)
	{		
		this.screenWriter.setColor(Color.WHITE);
		
		if(below == false)
		{
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY()-this.height-this.screenWriter.getHeight()-5));	
		}		
		else
		{
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY() + 5));
		}
	}
	*/
	
	
}
