package gameobjects.characters;

import gameaction.AbstractGameAction;
import gameaction.GameActionScheduler;
import gameobjects.AbstractGameObject;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.util.List;

import state.GameState;

import location.Hotspot;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;

/**
 * This class is holding basic animations that every game character needs.
 * @author Frank Bruns
 *
 */
public abstract class AbstractCharacterSprite extends AbstractGameObject
{	
	public static final int DIR_NO_DIR = 0;
	public static final int DIR_LEFT = 1;
	public static final int DIR_RIGHT = 2;
	private int currDirection = DIR_NO_DIR;
	private int prevDirection = -1;
	
	private double targetPosX = 0;
	private double targetPosY = 0;
		
	Color speechColor = null;	
	private boolean justUsedExit = false;
		
	private GameActionScheduler gameActionScheduler = null;
	
	
	/**
	 * Creates a new character sprite for a game character
	 * @param name Name of the character
	 * @param bshName name that will be used for this character in
	 * beanshell scripts
	 * set to the character at first. Use it like [YourCharacterClass].WALK_LEFT
	 * @param x x-coordinate, where the sprite should initially appear
	 * @param y y-coordinate, where the sprite should initially appear
	 * @param speechColor color for the spoken dialog lines on screen
	 * @param bsLoader
	 * @param bsInput
	 * @param bsSound
	 */
	public AbstractCharacterSprite(String name, String bshName, double x, double y, Color speechColor, BaseLoader bsLoader, BaseInput bsInput, BaseAudio bsSound)
	{
		super(name, bshName, x, y, bsLoader, bsInput, bsSound);
		
		this.gameActionScheduler = new GameActionScheduler(this);		
		this.speechColor = speechColor;	
		//this.sceneHotspot = new Hotspot(name,this.getX(),this.getY(),this.getWidth(),this.getHeight(),bsInput);
	}
	
	/**
	 * Sets the current direction to the specified constant.
	 * E.g. <code>AbstractCharacterSprite.DIR_LEFT</code>
	 * @param dir constant defning the direction of the character
	 */
	public void setDirection(int dir)
	{
		this.prevDirection = this.currDirection;
		this.currDirection = dir;
	}
	
	/**
	 * Returns the current set direction value as a constant like
	 * <code>AbstractCharacterSprite.DIR_RIGHT</code>
	 * @return current direction value
	 */
	public int getDirection()
	{
		return this.currDirection;
	}
	
	/**
	 * Return the direction that was set before a new direction was set
	 * with setDirection()
	 * @return direction before the current one
	 */
	public int getPreviousDirection()
	{
		return this.prevDirection;
	}
	
	/**
	 * Sets the new target position for the character
	 * @param x x-coordinate
	 * @param y y-coordinate
	 */
	public void setTargetPosition(double x, double y)
	{
		this.targetPosX = x;
		this.targetPosY = y;
	}
	
	/**
	 * Sets the new target x-coordinate for the character
	 * @param x x-coordinate
	 */
	public void setTargetX(double x)
	{
		this.targetPosX = x;
	}

	/**
	 * Sets the new target y-coordinate for the character
	 * @param y y-coordinate
	 */
	public void setTargetY(double y)
	{
		this.targetPosY = y;
	}
	
	/**
	 * Returns the x-coordinate of the sprite's target position
	 * @return x-coordinate of target position
	 */
	public double getTargetX()
	{
		return this.targetPosX;
	}
	
	/**
	 * Returns the y-coordinate of the sprite's target position
	 * @return y-coordinate of target position
	 */
	public double getTargetY()
	{
		return this.targetPosY;
	}
	
	/**
	 * Returns the speech color of this character
	 * @return speech color of this character
	 */
	public Color getSpeechColor()
	{
		return this.speechColor;
	}
	
	/**
	 * Returns the game action scheduler
	 * @return game action scheduler
	 */
	public GameActionScheduler getGameActionScheduler()
	{
		return this.gameActionScheduler;
	}
	
	/**
	 * Sets if the character just used an exit. This is important to set after using
	 * an exit, because the mouse position must not be processed just after
	 * using an exit. 
	 * @param usedExit player just used an exit? --> true=yes, false=no
	 */
	public synchronized void setJustUsedExit(boolean usedExit)
	{
		this.justUsedExit = usedExit;
	}
	
	/**
	 * Determines whether the character has just used an exit
	 * @return has the player just used an exit? true=yes, false=no
	 */
	public synchronized boolean hasJustUsedExit()
	{
		return this.justUsedExit;
	}
	
	/**
	 * Convenience method to add a game action to the characters
	 * game action scheduler without calling the getGameActionScheduler()
	 * method first.
	 * @param action game action to add to the game action manager
	 */
	public synchronized void addAction(AbstractGameAction action)
	{
		this.gameActionScheduler.add(action);
	}
	
	/**
	 * Convenience method to add a list of game actions to the characters
	 * game action scheduler at once without calling the getGameActionScheduler()
	 * method first.
	 * @param actions game actions to add to the game action manager
	 */
	public synchronized void addActions(List<AbstractGameAction> actions)
	{
		for(AbstractGameAction ga : actions) this.gameActionScheduler.add(ga);
	}
	
	/**
	 * Convenience method that pauses the game action manager of the character
	 *
	 */
	public void pause()
	{
		this.gameActionScheduler.pause();
	}
	
	/**
	 * Convenience method that resumes the game action manager of the character
	 *
	 */
	public void resume()
	{
		this.gameActionScheduler.resume();
	}
	
	@Override
	public void render(Graphics2D g)
	{
		super.render(g);
		this.gameActionScheduler.render(g);
	}

	@Override
	public void update(long elapsedTime)
	{
		this.gameActionScheduler.update(elapsedTime);
		super.update(elapsedTime);				
	}
}
