package gameobjects.item;

import gameobjects.AbstractGameObject;
import gameobjects.characters.Player;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.*;
import location.Hotspot;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;

/**
 * This class represents an item in the game. Items are objects
 * with which the player can interact in some way.
 * @author Frank Bruns
 *
 */
public class Item extends AbstractGameObject
{
	private Point inventoryPos = null;
	
	private String name = "no name yet";
	private Hotspot inventHotspot = null;

	private BufferedImage imageInventory = null;
	
	/** can the player take the item? */
	private boolean isPickable = false;
	private boolean isVisible = false;
	
	/**
	 * has the player issued the order to pick the item, so that
	 * it can be picked as soon as the player gets near to it?
	 */
	private boolean isNotedForPicking = false;
	
	/**
	 * Creates a new item
	 * @param name name of the item
	 * @param bshName name that will be used for this item in
	 * beanshell scripts
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param pickable is the item pickable by the player?
	 * @param inventImage image for the inventory
	 * @param bsLoader
	 * @param bsInput
	 * @param bsSound
	 */
	public Item(String name, String bshName, double x, double y, boolean pickable, boolean visible, BufferedImage inventImage,BaseLoader bsLoader, BaseInput bsInput, BaseAudio bsSound)
	{
		super(name,bshName,x,y, bsLoader, bsInput, bsSound);
				
		this.imageInventory = inventImage;
		
		this.inventoryPos = new Point();
		this.name = name;
		this.isPickable = pickable;
		this.isVisible = visible;
		
		// deactivate scene hotspot if item is not pickable
		if(this.isPickable == false) this.sceneHotspot.setShowName(false);
		
		//this.sceneHotspot = new Hotspot(name,this.getX(),this.getY(),this.getWidth(),this.getHeight(),bsInput);
		this.inventHotspot = new Hotspot(name,this.inventoryPos.x,
				this.inventoryPos.y,this.imageInventory.getWidth(),
				this.imageInventory.getHeight(),bsInput);
	}

	/**
	 * Returns the current position of the item in the inventory. This method
	 * returns <code>null</code> if the item is not pickable and hence could not
	 * have become a member of the inventory.
	 * @return position in the inventory area (x,y)-coordinates
	 */
	public Point getInventPosition()
	{
		if(this.isPickable == true) return this.inventoryPos;
		else return null;
	}
	
	/**
	 * Sets the inventory position of the item to the specified point
	 * @param point new position in the inventory
	 */
	public void setInventPosition(Point point)
	{
		this.inventoryPos = point;
	}
	
	/**
	 * Returns the image that represents the item in the inventory
	 * @return The item's inventory image
	 */
	public BufferedImage getInventoryImage()
	{
		return this.imageInventory;
	}
	
	/**
	 * Returns the hotspot of the item in the inventory
	 * @return hotspot of the item in the inventory
	 */
	public Hotspot getInventHotspot()
	{
		return this.inventHotspot;
	}
	
	/**
	 * Determines whether the item can be picked by the player
	 * @return can the player pick the item? --> true=yes, false=no
	 */
	public boolean isPickable()
	{
		return this.isPickable;
	}
	
	/**
	 * Sets whether the item is pickable
	 * @param pickable is item pickable? --> true=yes, false=no
	 */
	public void setPickable(boolean pickable)
	{
		this.isPickable = pickable;
		this.sceneHotspot.setShowName(pickable);
	}
	
	/**
	 * Determines whether the item shall be visible
	 * @return is the item visivle? --> true=yes, false=no
	 */
	public boolean isVisible()
	{
		return this.isVisible;
	}
	
	/**
	 * Sets whether the item ought to be visible
	 * @param pickable shall item be visible? --> true=yes, false=no
	 */
	public void setVisible(boolean visible)
	{
		this.isVisible = visible;
	}
	
	/**
	 * Sets if the item is noted for picking as soon as the player gets near
	 * the item
	 * @param noted note the item for later picking --> true=yes, false=no
	 */
	public void setNotedForPicking(boolean noted)
	{
		this.isNotedForPicking = noted;
	}
	
	/**
	 * Determines whether the item is noted for later picking, so that the
	 * item can be picked as soon as the player gets near to it
	 * @return is item noted for later picking? --> true=yes, false=no
	 */
	public boolean isNotedForPicking()
	{
		return this.isNotedForPicking;
	}
	
	/**
	 * Determines whether the item is within picking range of the
	 * player character
	 * @param player player object
	 * @return is item in picking range of the player? true=yes, false=no
	 */
	public boolean isInPickingRange(Player player)
	{	
		if(this.getHotspot().isPointInHotspot(player.getX(),player.getY())
				|| this.getHotspot().isPointInHotspot(player.getX()+player.getWidth(),player.getY()))
		{
			return true;
		}
		else return false;		
	}
	
	/**
	 * Shows the name of the inventory item right below it.
	 * @param g
	 */
	public void showNameinInventory(Graphics2D g)
	{		
		this.screenWriter.setColor(Color.WHITE);
		
			this.screenWriter.drawString(g, this.getName(),this.getInventPosition().x, 
					this.getInventPosition().y+this.getInventoryImage().getHeight()+5);		
	}
	
	@Override
	public void update(long elapsedTime)
	{
		super.update(elapsedTime);
		//this.inventHotspot.update(this.getX(), this.getY(), this.getWidth(), this.getHeight());
	}

}
