package inventory;

import gameobjects.item.Item;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.*;
import state.GameState;
import com.golden.gamedev.engine.BaseInput;

/**
 * This class represents the players inventory.
 * @author Frank Bruns
 *
 */
public class Inventory implements Serializable
{
	private LinkedList<Item> inventoryItems = null;	
	private Item selectedItem = null;
	
	private BaseInput bsInput = null;
	private int width = 0;
	private int height = 0;
	private boolean lookingAtInventory = false;
	
	/**
	 * Creates an inventory area that can store and display items.
	 * The origin of the area will be the upper left corner of the display.
	 * @param width width of the inventory area
	 * @param height height of the inventory area
	 * @param bsInput
	 */
	public Inventory(int width, int height, BaseInput bsInput)
	{
		this.width = width;
		this.height = height;
		this.bsInput = bsInput;
		this.inventoryItems = new LinkedList<Item>();
	}

	/**
	 * Returns a list with the items in the players inventory
	 * @return list of player inventory items
	 */
	public LinkedList<Item> getInventoryItems()
	{
		return inventoryItems;
	}
	
	/**
	 * Adds an item to the inventory
	 * @param item Item that shall be added to the inventory
	 */
	public void addItemToInventory(Item item)
	{
		this.inventoryItems.add(item);
	}
	
	/**
	 * Removes an item from the inventory. If there is a concurrent modification
	 * exception while calling this method, please consider to add the item
	 * to a remove-list and remove the item in the update method instead.
	 * @param item item to remove from the inventory
	 */
	public void removeItem(Item item)
	{
		this.inventoryItems.remove(item);
	}
	
	/**
	 * Determines whether the player is currently looking at the inventory.
	 * For example, this is important for player movement, because the player
	 * shall not take a new target location while clicking on an inventory item.
	 * @return is player currently looking at inventory? --> true=yes, false=no
	 */
	public boolean isLookingAtInventory()
	{
		return this.lookingAtInventory;
	}
	
	/**
	 * Returns the currently selected inventory item. 
	 * @return the currently selected inventory item, null if none is
	 * selected
	 */
	public Item getSelectedInventoryItem()
	{
		return this.selectedItem;
	}
	
	/**
	 * Determines whether the inventory contains the specified item
	 * @param item item to check for existence in the inventory
	 * @return inventory contains the item? --> true=yes, false=no
	 */
	public boolean hasItem(Item item)
	{
		if(this.inventoryItems.contains(item)) return true;
		else return false;
	}
	
	/**
	 * Sets the currently selected inventory item to the specified item.
	 * Set it to null to express that no imventory item is selected
	 * @param inventItem item to set to the seleceted inventory item
	 */
	public void setSelectedInventoryItem(Item inventItem)
	{
		this.selectedItem = inventItem;
	}
	
	/**
	 * Draws the inventory area with all its items if the mouse is located 
	 * within the area's bounds
	 * @param g
	 */
	public void render(Graphics2D g)
	{
		if(GameState.equals(GameState.NORMAL) ==  true)
		{			
			// check, if the mouse is within the bounds of the inventory area
			if((this.bsInput.getMouseX() >= 0 && this.bsInput.getMouseX() <= this.width)
					&& (this.bsInput.getMouseY() >= 0 && this.bsInput.getMouseY() <= this.height))
			{				
				// signal that the player is looking at the inventory
				this.lookingAtInventory = true;								
				
				// draw the inventory area
				this.renderInventoryItems(g);
			}
			else
			{	
				// signal that the player is not looking at the inventory anymore
				if(this.lookingAtInventory == true)	this.lookingAtInventory = false;
			}	
		}		
	}
	
	/**
	 * Updates the inventory items, especially their hotspots in the
	 * inventory area
	 * @param elapsedTime time between two frames
	 */
	public void update(long elapsedTime)
	{
		for(Item item : this.inventoryItems)
		{
			item.getInventHotspot().update(item.getInventPosition().x,
					item.getInventPosition().y,item.getInventoryImage().getWidth(),
					item.getInventoryImage().getHeight());	
		}		
	}
	
	/**
	 * Helper method to calculate the offset for the invetory items.
	 * The returned value can then be used to calculate the exact position
	 * for the item in the inventory area.
	 * @return offset that can be used to calculate the item position in 
	 * the inventory area
	 */
	private int calculateItemOffset()
	{
		float offset = 0f;
		float totalWidth = 0f;
		for(Item iventoryItem : this.inventoryItems)
		{
			totalWidth += iventoryItem.getInventoryImage().getWidth();
		}
		offset = totalWidth /(float)this.inventoryItems.size();		
		
		return (int)offset+15; // 15px distance between the inventory items
	}
	
	/**
	 * Helper method to render the inventory area
	 * @param g
	 */
	private void renderInventoryItems(Graphics2D g)
	{
		// draw the rectangular area
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, this.width, this.height);
		
		// calculate offset for the items
		int offset = this.calculateItemOffset();
		
		for(Item inventoryItem : this.inventoryItems)
		{		
			inventoryItem.setInventPosition(this.calculateItemPosition(inventoryItem, offset));
						
			BufferedImage img = inventoryItem.getInventoryImage();
			g.drawImage(img, 
					inventoryItem.getInventPosition().x, 
					inventoryItem.getInventPosition().y,
					img.getWidth(), img.getHeight(), null);
			
			// draw description of the item if the mouse is over it
			if(inventoryItem.getInventHotspot().isMouseOverInventHotspot())
			{
				inventoryItem.showNameinInventory(g);
			}
		}
	}
	
	/**
	 * Helper method to calculate the exact position for the item
	 * in the inventory area
	 * @param inventoryItem the item to calculate the position for
	 * @param offset the offset that was previously calculated for inventory items
	 * @return
	 */
	private Point calculateItemPosition(Item inventoryItem, int offset)
	{
		int x = offset * this.inventoryItems.indexOf(inventoryItem);
		return new Point(x,5);
	}
}
