package location;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.Graphics2D;
import java.io.Serializable;

import state.GameState;

import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.object.font.SystemFont;

import cursor.CursorManager;
import cursor.Cursors;

/**
 * This class represents a hotspot in the game. Hotspots are areas of special
 * interest on the screen. This could be items, characters or just an
 * interesting place. Hotspots are represented by an invisible rectangle
 * defining the area of the hotspot
 * 
 * @author Frank Bruns
 * 
 */
public class Hotspot implements Serializable
{
	private double x = 0.0;

	private double y = 0.0;

	private int width = 0;

	private int height = 0;

	private String name = "no name yet";

	private boolean showName = true;
	
	private BaseInput bsInput = null;
	
	private SystemFont screenWriter = null;

	/**
	 * Creates a new hotspot with the specified parameters
	 * 
	 * @param name name that is to be shown on the screen for the hotspot
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param width width of the hotspot
	 * @param height height of the hotspot
	 * @param bsInput
	 */
	public Hotspot(String name, double x, double y, int width, int height,
			BaseInput bsInput)
	{
		this.name = name;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.bsInput = bsInput;
		this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,15),Color.WHITE);
	}
	
	/**
	 * Sets whether the name of the hotspot shall be displayed
	 * @param showName show the hotspot's name? --> true=yes, false=no
	 */
	public void setShowName(boolean showName)
	{
		this.showName = showName;
	}

	/**
	 * Returns the height of the hotspot area
	 * 
	 * @return height of the hotspot area
	 */
	public int getHeight()
	{
		return height;
	}

	/**
	 * Sets the height of the hotspot area to the specified height
	 * 
	 * @param height
	 *            height of the hotspot area
	 */
	public void setHeight(int height)
	{
		this.height = height;
	}

	/**
	 * Returns the name of the hotspot area
	 * 
	 * @return name of the hotspot area
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * Sets the name of the hotspot area
	 * 
	 * @param name
	 *            name of the hotspot area
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * Returns the width of the hotspot area
	 * 
	 * @return width of the hotspot area
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * Sets the width of the hotspot area to the specified width
	 * 
	 * @param width
	 *            width of the hotspot area
	 */
	public void setWidth(int width)
	{
		this.width = width;
	}

	/**
	 * Returns the x-coordinate of the hotspot area
	 * 
	 * @return x-coordinate of the hotspot
	 */
	public double getX()
	{
		return x;
	}

	/**
	 * Sets the x-coordinate of the hotspot area
	 * 
	 * @param x
	 *            x-coordinate of the hotspot area
	 */
	public void setX(double x)
	{
		this.x = x;
	}

	/**
	 * Returns the y-coordinate of the hotspot area
	 * 
	 * @return y-coordinate of the hotspot area
	 */
	public double getY()
	{
		return this.y;
	}

	/**
	 * Sets the y-coordinate of the hotspot area
	 * 
	 * @param y
	 *            y-coordinate of the hotspot area
	 */
	public void setY(double y)
	{
		this.y = y;
	}

	/**
	 * Updates the hotspot parameters in order to be always consistent with the
	 * scene, if the hotspot area can move
	 * 
	 * @param x
	 *            x-coordinate
	 * @param y
	 *            y-coordinate
	 * @param width
	 *            width of the hotspot area
	 * @param height
	 *            height of the hotspot area
	 */
	public void update(double x, double y, int width, int height)
	{
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		//System.out.println("[Hotspot] "+this.name+"  w: "+this.width+"  h:"+this.height);
	}
	
	/**
	 * Renders hotspot things
	 * @param g
	 */
	public void render(Graphics2D g)
	{						
		if(GameState.equals(GameState.NORMAL) ==  true)
		if (this.showName == true && this.isMouseOverHotspot())
		{
			this.showName(g, false);
		}
	}

	/**
	 * Determines whether the mouse is currently over the hotspot
	 * @return is mouse over hotspot? --> true=yes, false=no
	 */
	public boolean isMouseOverHotspot()
	{
		if (this.isPointInHotspot(this.bsInput.getMouseX(), this.bsInput.getMouseY()))
		{
			return true;
		}
		else return false;
	}
	
	/**
	 * Determines whether the mouse is currently over an inventory hotspot
	 * @return is mouse over inventory hotspot? --> true=yes, false=no
	 */
	public boolean isMouseOverInventHotspot()
	{
		if (this.isPointInInventHotspot(this.bsInput.getMouseX(), this.bsInput.getMouseY()))
		{
			return true;
		}
		else return false;
	}

	/**
	 * Determines whethter the specified coordinates are within the hotspot
	 * bounds
	 * 
	 * @param x
	 *            x-coordinate of the mouse
	 * @param y
	 *            y-coordinate of the mouse
	 * @return is point in hotspot? --> true=yes, false=no
	 */
	public boolean isPointInHotspot(double x, double y)
	{
		if ((x >= this.x && x <= this.x + this.width)
				&& (y >= this.y && y <= this.y + this.height))
		{
			return true;
		}
		else return false;
	}
	
	/**
	 * Determines whethter the specified coordinates are within the hotspot of
	 * the item in the inventory
	 * 
	 * @param x
	 *            x-coordinate of the mouse
	 * @param y
	 *            y-coordinate of the mouse
	 * @return is point in inventory hotspot? --> true=yes, false=no
	 */
	public boolean isPointInInventHotspot(double x, double y)
	{
		if ((x >= this.x && x <= this.x + this.width)
				&& (y >= this.y && y <= this.y + this.height))
		{
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Shows the name of the hotspot below or above it.
	 * @param g
	 * @param below show name below game object? -->true=yes, false=no
	 */
	public void showName(Graphics2D g, boolean below)
	{		
		this.screenWriter.setColor(Color.WHITE);
		g.setColor(Color.BLACK);
		Composite compBackUp = g.getComposite();
		
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, 0.55f));
		
		if(below == false)
		{			
			g.fillRect((int)(this.getX()), 
					(int)(this.getY()-this.screenWriter.getHeight()-5),
					this.screenWriter.getWidth(this.name),
					this.screenWriter.getHeight());
			// set alpha to old value
			g.setComposite(compBackUp);
					
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY()-this.screenWriter.getHeight()-5));	
		}		
		else
		{
			g.fillRect((int)(this.getX()), 
					(int)(this.getY()+this.getHeight()+5),
					this.screenWriter.getWidth(this.name),
					this.screenWriter.getHeight());
			
			// set alpha to old value
			g.setComposite(compBackUp);
			
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY()+ this.height + 5));
		}		
	}
}
