package location;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;

import state.GameState;

import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;
import com.golden.gamedev.object.font.SystemFont;

/**
 * This class represents an interactive hotspot. That is an extension to
 * a normal hotspot. Normal hotspot usually are used together with game
 * objects in order to make them sensitive to mouse movements. Now an 
 * interactive hotspot is a hotspot especially designed to exit stand alone
 * at the game location. That means there is no need to connect it to a
 * game object. The user can perform actions on the hotspot directly, for
 * example via beanshell scripts. The following example may explain this a
 * little further. Imagine a lock that can be opened by certain key. 
 * Because you don't want to take the overhead of placeing an item that 
 * represents the lock in a door, you can easily setup an interactive
 * hotspot at the place of the look to make it sensitive for actions by
 * the player.
 * @author Frank Bruns
 *
 */
public class InteractiveHotspot extends Hotspot
{	
	private String bshName = "no beanshell name yet";
	private boolean active = true;
	private BufferedImage image = null;
	private BaseLoader bsLoader = null;
	
	/**
	 * Creates a new hotspot with the specified parameters
	 * 
	 * @param name name that is to be shown on the screen for the 
	 * interactive hotspot
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param width width of the interactive hotspot
	 * @param height height of the interactive hotspot
	 * @param bsInput
	 * @param bsLoader
	 */
	public InteractiveHotspot(String name, String bshName, double x, double y, int width, int height, BaseInput bsInput, BaseLoader bsLoader)
	{
		super(name,x,y-height,width,height,bsInput);
		this.bshName = bshName;
		//this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,15),Color.WHITE);
		this.bsLoader = bsLoader;
	}
	
	/**
	 * Returns the beanshell name for this interactive hotspot
	 * @return name for adressing this hotspot in beanshell script files 
	 */
	public String getBshName()
	{
		return this.bshName;
	}
	
	/**
	 * Determines whether the interactive hotspot is active.
	 * @return is interactive hotpsot active? --> true=yes, false=no
	 */
	public boolean isActive()
	{
		return this.active;
	}
	
	/**
	 * Sets the interactive hotspot to inactive state. You can do this
	 * for example if the hotspot was used with the correct item and 
	 * isn't needed for explicit user interaction anymore
	 *
	 */
	public void setInactive()
	{
		this.active = false;
	}
	
	/**
	 * Sets the interactive hotspot to active state if it was not active
	 * before
	 *
	 */
	public void setActive()
	{
		if(this.active == false) this.active = true;
	}
	
	/**
	 * Sets the assigned image of the interactive hotspot to the
	 * specified image
	 * @param img image to assign to the interactive hotspot
	 */
	public void setImage(BufferedImage img)
	{
		this.image = img;
	}
	
	/**
	 * Sets the image assigned to the interactive hotspot
	 * @param imagePath path+filename of the image to set
	 */
	public void setImage(String imagePath)
	{
		this.image = bsLoader.getImage(imagePath);
	}

	/**
	 * Shows the name of the interactive hotspot below or above it.
	 * @param g
	 * @param below show name below hotspot? -->true=yes, false=no
	 */
	/*
	public void showName(Graphics2D g, boolean below)
	{		
		this.screenWriter.setColor(Color.WHITE);
		
		if(below == true)
		{
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY()-this.screenWriter.getHeight()-5));	
		}		
		else
		{
			this.screenWriter.drawString(g, this.getName(), (int)(this.getX()), 
					(int)(this.getY()+this.getHeight()+this.screenWriter.getHeight()+5));
		}
	}
	*/
	
	/**
	 * Renders the interactive hotspot name
	 * @param g
	 */
	public void render(Graphics2D g)
	{
		if(GameState.equals(GameState.NORMAL) ==  true)
		{
			if (this.isActive() && this.isMouseOverHotspot())
			{
				this.showName(g, false);
			}
		}
		
		if(this.image != null) g.drawImage(this.image,null,(int)this.getX(),(int)this.getY());
	}
}
