package location;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;

import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.NPCSprite;
import gameobjects.characters.Player;

/**
 * This class represents an exit from a game location
 * @author Frank Bruns
 *
 */
public class LocationExit extends Hotspot
{
	private GameLocation startLocation = null;
	private GameLocation destLocation = null;
	
	private String bshName = "no beanshell name yet";
	
	private boolean canBeUsed = true;
	private boolean isNotedForUse = false;
	
	private double newX = 0;
	private double newY = 0;
	
	
	/**
	 * 
	 * @param name name of the location exit
	 * @param bshName beanshell name for the location exit
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param width width of the location exit
	 * @param height height of the location exit
	 * @param newX new x-coordinate for the player
	 * @param newY new y-coordinate for the player
	 * @param usable determines if the exit can be initially used
	 * @param start game location the player can leave with the exit (start location)
	 * @param dest game location to which the exit leads to (destination)
	 * @param bsInput
	 */
	public LocationExit(String name, String bshName,double x, double y, int width, int height, double newX, double newY, boolean usable, GameLocation start, GameLocation dest, BaseInput bsInput)
	{
		super(name,x,y-height,width,height,bsInput);
		this.bshName = bshName;
		this.startLocation = start;
		this.destLocation = dest;
		this.newX = newX;
		this.newY = newY;
		this.canBeUsed = usable;
	}
	
	/**
	 * Return the beanshell name of this location exit
	 * @return beanshell name of this location exit
	 */
	public String getBshName()
	{
		return this.bshName;
	}
	
	/**
	 * Returns the location from which the exit leads to new location. That is the
	 * start location of the exit.
	 * @return start game location 
	 */
	public GameLocation getStartLocation()
	{
		return this.startLocation;
	}
	
	/**
	 * Returns the location to which the location exit leads to. This will be the
	 * next location to visit, if the exit is used
	 * @return destination game location
	 */
	public GameLocation getDestinationLocation()
	{
		return this.startLocation;
	}
	
	/**
	 * Determines whether the exit can be used
	 * @return can the exit be used? --> true=yes, false=no
	 */
	public boolean isUsable()
	{
		return this.canBeUsed;
	}
	
	/**
	 * Sets if the exit can be used
	 * @param usable is exit usable? true=yes, false=no
	 */
	public void setUsable(boolean usable)
	{
		this.canBeUsed = usable;
	}
	
	/**
	 * Sets whether the exit was noted for use by the player
	 * @param use use the exit? true=yes, false=no
	 */
	public void setNotedForUse(boolean use)
	{
		this.isNotedForUse = use;
	}
	
	/**
	 * Returns whether the exit is noted for use
	 * @return is exit noted for use? true=yes, false=no
	 */
	public boolean isNotedForUse()
	{
		return this.isNotedForUse;
	}
	
	/**
	 * Determines whether the exit is in range for a character
	 * @param character character that is checked for the range to the exit
	 * @return is exit in range for the player? true=yes, false=no
	 */
	public boolean isInRange(AbstractCharacterSprite character)
	{		
		if(this.isPointInHotspot(character.getX(),character.getY())
				|| this.isPointInHotspot(character.getX()+character.getWidth(),character.getY())
				//|| this.isPointInHotspot(character.getX()+character.getWidth()/2,character.getY()-character.getHeight())
				)
		{
			return true;
		}
		else return false;		
	}
	
	/**
	 * Call this method to signal that the character wants to make a transition from
	 * this exit's start location to its destination location. Basically everything this 
	 * method does is to set the start location to "not active" and the 
	 * destination location to "active" by calling the locations' setActive() methods.
	 * Furthermore the player gets removed from the start location and get added to
	 * the destination location
	 * @param character character that uses the exit
	 */
	public synchronized void use(AbstractCharacterSprite character)
	{
		// reset the noted for use flag
		this.isNotedForUse = false;
		
		if(character instanceof Player)
		{
			Player player = (Player)character;
			player.getInventory().setSelectedInventoryItem(null);
			this.startLocation.setActive(false);
			this.startLocation.removePlayer();
			this.destLocation.addPlayer((Player)character);
			this.destLocation.setActive(true);
			
			// change background music if the destination 
			// has another musical background theme
			if(this.destLocation.getMusicFileName().equals(this.startLocation.getMusicFileName()) == false)
			{
				this.destLocation.playMusic(true);
			}
		}
		else
		{
			this.startLocation.removeCharacter((NPCSprite)character);
			this.destLocation.addCharacter((NPCSprite)character);
		}
		
		character.setTargetPosition(this.newX, this.newY);
		character.setLocation(this.newX, this.newY);		
		character.setJustUsedExit(true);				
	}
}
