package path;

import java.io.Serializable;
import java.util.List;

/**
 * The AStarNode class, along with the AStarSearch class, implements a generic
 * A* search algorthim. The AStarNode class should be subclassed to provide
 * searching capability.
 * 
 * @author David Brackeen (Book: Developing games in Java)
 */
public abstract class AStarNode implements Serializable, Comparable
{

	AStarNode pathParent;

	float costFromStart;

	float estimatedCostToGoal;

	public float getCost()
	{
		return costFromStart + estimatedCostToGoal;
	}

	public int compareTo(Object other)
	{
		float otherValue = ((AStarNode) other).getCost();
		float thisValue = this.getCost();

		return this.sign(thisValue - otherValue);
	}

	/**
	 * Gets the cost between this node and the specified adjacent (aka
	 * "neighbor" or "child") node.
	 */
	public abstract float getCost(AStarNode node);

	/**
	 * Gets the estimated cost between this node and the specified node. The
	 * estimated cost should never exceed the true cost. The better the
	 * estimate, the more effecient the search.
	 */
	public abstract float getEstimatedCost(AStarNode node);

	/**
	 * Gets the children (aka "neighbors" or "adjacent nodes") of this node.
	 */
	public abstract List getNeighbors();

	/**
	 * Returns the sign of the number. Returns -1 for negative, 1 for positive,
	 * and 0 otherwise.
	 */
	public int sign(float v)
	{
		return (v > 0) ? 1 : (v < 0) ? -1 : 0;
	}
}
