package path;

import java.util.ArrayList;
import java.util.List;

/**
 * Node that represents a square in the walkable area grid. This node is then
 * used for A* path finding.
 * @author Frank Bruns
 *
 */
public class GridNode extends AStarNode
{	
	private double x = -1;
	private double y = -1;
	private int width = -1;
	private int height = -1;
	boolean walkable = true;
	
	private ArrayList<GridNode> neighbours = null;
	
	/**
	 * Creates a new grid node with the specified parameters
	 * @param x x coordinate of the grid node
	 * @param y y coordinate of the grid node
	 * @param width width of the grid node
	 * @param height height of the grid node
	 */
	public GridNode(double x, double y, int width, int height)
	{
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.neighbours = new ArrayList<GridNode>();
	}
	
	/**
	 * Sets the walkable status for the grid node
	 * @param walkable set walkable? --> true=yes, false=no
	 */
	public void setWalkable(boolean walkable)
	{
		this.walkable = walkable;
	}
	
	/**
	 * Returns the x coordinate of the center point of the grid node
	 * @return center point's x coordinate
	 */
	public double getMidX()
	{
		return (this.x+(double)this.width)/2.0;
	}
	
	/**
	 * Returns the y coordinate of the center point of the grid node
	 * @return center point's y coordinate
	 */
	public double getMidY()
	{
		return (this.y+(double)this.height)/2.0;
	}
	
	/**
	 * Checks, if the specified coordinates lay in this grid node's bounds
	 * @param x x coordinates
	 * @param y y coordinates
	 * @return are coordinates within grid node's bounds? --> true=yes, false=no
	 */
	public boolean containsPoint(double x, double y)
	{
		if ((x >= this.x && x < this.x + this.width)
				&& (y >= this.y && y < this.y + this.height))
		{
			return true;
		}
		else return false;
	}
	
	/**
	 * Determines whether the grid node is a neighbour of the specified node.
	 * @param node node to check, if naighbour
	 * @return is neighbour? --> true=yes, false=no
	 */
	public boolean isNeighbor(GridNode node)
	{
		// right now no diagonal nodes are considered
		if((node.containsPoint(this.x-1,this.y))
				|| (node.containsPoint(this.x,this.y-1))
				//|| (node.containsPoint(this.x-1,this.y-1))
				//|| (node.containsPoint(this.x-1, this.y+this.height+1))
				|| (node.containsPoint(this.x+this.width+1,this.y))
				//|| (node.containsPoint(this.x+this.width+1,this.y-1))
				|| (node.containsPoint(this.x,this.y+this.height+1)))
				//|| (node.containsPoint(this.x+this.width+1,this.y+this.height+1 )))
			return true;
		else return false;
	}
	
	/**
	 * Returns the height of the grid node
	 * @return height of the grid node
	 */
	public int getHeight()
	{
		return height;
	}

	/**
	 * Returns the width of the grid node
	 * @return width of the grid node
	 */
	public int getWidth()
	{
		return width;
	}

	/**
	 * Returns the x coordinate of the grid node
	 * @return x coordinate of the grid node
	 */
	public double getX()
	{
		return x;
	}

	/**
	 * Returns the y coordinate of the grid node
	 * @return y coordinate of the grid node
	 */
	public double getY()
	{
		return y;
	}
	
	/**
	 * Determines whether the grid node is walkable
	 * @return is grid node walkable? --> true=yes, false=no
	 */
	public boolean isWalkable()
	{
		return this.walkable;
	}
	
	/**
	 * Adds a neighbour to the list of neighbours of this grid node
	 * @param node grid node neighbour node to add
	 */
	public void addNeighbor(GridNode node)
	{
		this.neighbours.add(node);
	}

	@Override
	public float getCost(AStarNode node)
	{
		return this.getEstimatedCost(node);
	}

	@Override
	public float getEstimatedCost(AStarNode node)
	{
		if(this.walkable == true) return 1f;
		else return 1000f;
	}

	@Override
	public List<GridNode> getNeighbors()
	{
		return this.neighbours;
	}
}
