package path;

import gameobjects.item.Item;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.ArrayList;

import location.Hotspot;
import location.InteractiveHotspot;
import location.LocationExit;


/**
 * This class represents the grid that contains the walkable area of a
 * game location
 * @author Frank Bruns
 *
 */
public class SearchGrid implements Serializable
{
	private BufferedImage imgWalkableArea = null;
	private ArrayList<Item> items = null;
	private ArrayList<InteractiveHotspot> ihspots = null;
	private ArrayList<LocationExit> exits = null;
	private int gridNodeSize = -1;
	private GridNode[][] gridNodes = null;	
	
	/**
	 * Creates a new search grid out of the specified walkable area image
	 * @param imgWalkable image containing the walkable area
	 * @param items items of the game location of the search grid
	 * @param ihs interactive hotspots of the game location of the search grid
	 * @param exits location exits of the game location of the search grid
	 * @param gridNodeSize width and height for the square of a grid node, this
	 * must be a power of 2 
	 */
	public SearchGrid(BufferedImage imgWalkable, ArrayList<Item> items, ArrayList<InteractiveHotspot> ihs, ArrayList<LocationExit> exits, int gridNodeSize)
	{		
		this.imgWalkableArea = imgWalkable;
		this.items = items;				
		this.ihspots = ihs;
		this.exits = exits;
		this.gridNodeSize = gridNodeSize;
		this.gridNodes = new GridNode[imgWalkable.getHeight()/gridNodeSize][imgWalkable.getWidth()/gridNodeSize];
		this.createGridNodes();
		this.calculateNeighbours();
	}
	
	/**
	 * Returns the size of a grid node square
	 * @return size of a grid node square
	 */
	public int getGridNodeSize()
	{
		return this.gridNodeSize;
	}
	
	/**
	 * Fills the search grid with grid nodes (squares). The walkable status
	 * of the grid node is set too. 
	 *
	 */
	private void createGridNodes()
	{		
		// y-line
		for(int i=0; i<this.imgWalkableArea.getHeight()/gridNodeSize; i++)
		{						
			// x-line
			for(int j=0; j<this.imgWalkableArea.getWidth()/gridNodeSize; j++)
			{
				this.gridNodes[i][j] = new GridNode(j*gridNodeSize,i*gridNodeSize,this.gridNodeSize,this.gridNodeSize);				
				
				// set walkable status of grid node
				int gridNodeMidX = ((int)this.gridNodes[i][j].getX()+this.gridNodes[i][j].getWidth()/2);
				int gridNodeMidY = ((int)this.gridNodes[i][j].getY()+this.gridNodes[i][j].getHeight()/2);
				
				// set walkable status with walkable image
				if(this.imgWalkableArea.getRGB(gridNodeMidX, gridNodeMidY) == Color.WHITE.getRGB())
				{
					this.gridNodes[i][j].setWalkable(false);
				}
				
				// set walkabe status with hotspot areas
				// check if a corner of the grid node is within the hotspot
				int gnUpperLeftX = (int)this.gridNodes[i][j].getX();
				int gnUpperLeftY = (int)this.gridNodes[i][j].getY();
				
				int gnLowerLeftX = (int)this.gridNodes[i][j].getX();
				int gnLowerLeftY = (int)this.gridNodes[i][j].getY()+this.gridNodes[i][j].getHeight();
				
				int gnUpperRightX = (int)this.gridNodes[i][j].getX()+this.gridNodes[i][j].getWidth();
				int gnUpperRightY = (int)this.gridNodes[i][j].getY();
				
				int gnLowerRightX = (int)this.gridNodes[i][j].getX()+this.gridNodes[i][j].getWidth();
				int gnLowerRightY = (int)this.gridNodes[i][j].getY()+this.gridNodes[i][j].getHeight();
				
				for(Item item : this.items)
				{												
					if(item.getHotspot().isPointInHotspot(gnUpperLeftX,gnUpperLeftY)
							|| item.getHotspot().isPointInHotspot(gnLowerLeftX,gnLowerLeftY)
							|| item.getHotspot().isPointInHotspot(gnUpperRightX,gnUpperRightY)
							|| item.getHotspot().isPointInHotspot(gnLowerRightX,gnLowerRightY))
					{						
						this.gridNodes[i][j].setWalkable(true);						
					}					
				}
				for(InteractiveHotspot ihs : this.ihspots)
				{					
					if(ihs.isPointInHotspot(gnUpperLeftX,gnUpperLeftY)
							|| ihs.isPointInHotspot(gnLowerLeftX,gnLowerLeftY)
							|| ihs.isPointInHotspot(gnUpperRightX,gnUpperRightY)
							|| ihs.isPointInHotspot(gnLowerRightX,gnLowerRightY))
					{						
						this.gridNodes[i][j].setWalkable(true);						
					}
				}
				
				for(LocationExit exit : this.exits)
				{					
					if(exit.isPointInHotspot(gnUpperLeftX,gnUpperLeftY)
							|| exit.isPointInHotspot(gnLowerLeftX,gnLowerLeftY)
							|| exit.isPointInHotspot(gnUpperRightX,gnUpperRightY)
							|| exit.isPointInHotspot(gnLowerRightX,gnLowerRightY))
					{						
						this.gridNodes[i][j].setWalkable(true);						
					}
				}
			}
		}
	}
	
	/**
	 * Returns the 2 dimensional array of grid nodes representing the
	 * search grid. The first dimension is the y dimension and the second one
	 * is the x dimension
	 * @return search grid nodes [y][x]
	 */
	public GridNode[][] getGridNodes()
	{
		return this.gridNodes;
	}
	
	/**
	 * Returns the image containing the walkable area of the search grid
	 * @return image containing the walkable area
	 */
	public BufferedImage getImageWalkable()
	{
		return this.imgWalkableArea;
	}
	
	/**
	 * Calculates the neighbour grid nodes for each node of the search grid
	 *
	 */
	private void calculateNeighbours()
	{
		// y line 
		for(int i=0; i<this.gridNodes.length; i++)
		{						
			// x line
			for(int j=0; j<this.gridNodes[i].length; j++)
			{
				for(int m=0; m<this.gridNodes.length; m++)
				{						
					for(int n=0; n<this.gridNodes[m].length; n++)
					{
						if(gridNodes[i][j].isNeighbor(gridNodes[m][n]))
							gridNodes[m][n].addNeighbor(gridNodes[i][j]);
					}
				}
			}
		}
	}
	
	/**
	 * Returns the grid node at the specified coordinates
	 * @param x x coordinate
	 * @param y y coordinate
	 * @return grid node at the specified coordinates
	 */
	public GridNode getGridNode(double x, double y)
	{
		if(this.gridNodes != null)
		{
			for(int i=0; i<this.gridNodes.length; i++)
			{						
				for(int j=0; j<this.gridNodes[i].length; j++)
				{
					if(gridNodes[i][j].containsPoint(x, y)) return gridNodes[i][j];
				}
			}
		}
		return null;
	}
}
