package speech;

import gameaction.Talk;
import gameobjects.characters.Player;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.util.HashMap;

import state.GameState;

import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.object.font.SystemFont;

/**
 * This class represents the area that displays the questions for
 * a dialog.
 * @author Frank Bruns
 *
 */
public class QuestionSelectionArea implements Serializable
{
	private HashMap<String, Dialog> dialogSet = null;
	private Dialog dialog = null;
	private SystemFont screenWriter = null;
	private BaseInput bsInput = null;
	private Player player = null;
	
	private static final int START_X = 30;
	private static final int START_Y = 500;
	
	/**
	 * The dialog of that the questions shall be displayed in the
	 * question selection area
	 * @param player player object
	 * @param bsInput
	 */
	public QuestionSelectionArea(Player player, BaseInput bsInput)
	{
		this.player = player;
		this.bsInput = bsInput;
		this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,20),Color.WHITE);
		this.dialogSet = new HashMap<String, Dialog>();
	}
	
	/**
	 * Adds a dialog to the set of dialogs for the player
	 * @param dialog dialog to add to the set of dialogs
	 */
	public void addDialog(Dialog dialog)
	{
		this.dialogSet.put(dialog.getName(), dialog);
	}
	
	/**
	 * Sets the dialog for the question selection area
	 * @param dialogName dialog to set identified by its name
	 */
	public void setDialog(String dialogName)
	{
		this.dialog = this.dialogSet.get(dialogName);
	}
	
	/**
	 * Returns the dialog with the specified name
	 * @param name identifying name of the sought dialog
	 * @return dialog with the specified name
	 */
	public Dialog getDialog(String name)
	{
		return this.dialogSet.get(name);
	}
	
	/**
	 * Updates the question selection area, especially checks for a picked 
	 * question
	 * @param elapsedTime time between two frames
	 */
	public void update(long elapsedTime)
	{
		if(GameState.equals(GameState.DIALOG_SELECTION) && this.player.hasArrivedAtTarget())
		{
			this.checkForChosenQuestion();
		}
	}
	
	/**
	 * Renders the question selection area and the according questions
	 * to screen
	 * @param g
	 */
	public void render(Graphics2D g)
	{		
		if(GameState.equals(GameState.DIALOG_SELECTION) && this.player.hasArrivedAtTarget())
		{			
			this.renderQSARectangle(g);
			this.renderQuestions(g);
		}		
	}
	
	/**
	 * Renders the questions of dialog
	 * @param g
	 */
	private void renderQuestions(Graphics2D g)
	{
		if(this.dialog != null && this.dialog.getBranches().isEmpty() == false)
		{
			int offset = 0;
			for(DialogBranch branch : this.dialog.getBranches())
			{
				if(branch.isExcluded() == false)
				{
					int lineStart = START_Y+offset*this.screenWriter.getHeight();
					int lineEnd = START_Y+offset*this.screenWriter.getHeight()+this.screenWriter.getHeight();
					if((bsInput.getMouseY() > lineStart )
							&& bsInput.getMouseY() < lineEnd)
					{
						this.screenWriter.setColor(Color.YELLOW);
					}
					else this.screenWriter.setColor(Color.WHITE);
						
					this.screenWriter.drawString(g,branch.getAssociatedQuestion(), 
							START_X, lineStart);
					offset++;					
				}				
			}
		}
	}
	
	/**
	 * This method draws an outline around the question 
	 * selection area
	 * @param g
	 */
	private void renderQSARectangle(Graphics2D g)
	{
		if(this.dialog != null)
		{
			// find the width of the rectangle by finding the
			// longest question string
			int width = -1;
			for(DialogBranch branch : this.dialog.getBranches())
			{
				int questionWidth = this.screenWriter.getWidth(branch.getAssociatedQuestion()); 
				if(questionWidth > width)
				{
					width = questionWidth;
				}
			}
			
			// find height of the question selection area
			int height = this.screenWriter.getHeight() * this.dialog.getBranches().size();
			
			g.setColor(Color.YELLOW);		
			g.drawRect(
					START_X-10, 
					START_Y-5, 
					width+20, 
					height+this.screenWriter.getHeight()+5);
		}
	}
	
	/**
	 * Checks if the player picked a question and starts the according
	 * dialog branch if so.
	 *
	 */
	private void checkForChosenQuestion()
	{
		if(this.dialog != null && this.dialog.getBranches().isEmpty() == false)
		{
			int offset = 0;
			for(DialogBranch branch : this.dialog.getBranches())
			{
				if(branch.isExcluded() == false)
				{
					int lineStart = START_Y+offset*this.screenWriter.getHeight();
					int lineEnd = START_Y+offset*this.screenWriter.getHeight()+this.screenWriter.getHeight();
					if((bsInput.getMouseY() > lineStart )
							&& bsInput.getMouseY() < lineEnd)
					{					
						if(bsInput.getMousePressed() == MouseEvent.BUTTON1)
						{
							this.dialog.runDialogBranch(branch.getID());
							
							// check if the branch is an aborter branch,
							// if so set the dialog object to null
							Talk talkAction = (Talk)player.getGameActionScheduler().getFirst();
							if(talkAction.isAborter() == true) this.dialog = null;
						}
					}
					offset++;
				}				
			}
		}
	}
}
