package speech;

import gameobjects.characters.AbstractCharacterSprite;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.io.Serializable;

import state.GameState;

import com.golden.gamedev.engine.BaseAudioRenderer;
import com.golden.gamedev.object.GameFont;
import com.golden.gamedev.object.Timer;
import com.golden.gamedev.object.font.SystemFont;

/**
 * This class represents a single sentence
 * @author Frank Bruns
 *
 */
public class Sentence implements Serializable
{	
	private SystemFont screenWriter = null;
	
	private Timer timer = null;
	private String line = "";
	private int lineWidth = 0;
	private Color color = null;
	private AbstractCharacterSprite character = null;
	private String soundPath = null; 
	
	private int heightPixels = 0;
	
	private boolean sayNow = true;
	private boolean soundRunning = false;
	
	/**
	 * Creates a new sentence with the specified parameters
	 * @param character the character that says the sentence
	 * @param line the line to be spoken
	 * @param lineWidth width of a line until line wrap
	 * @param color the color to display the sentence with
	 * @param time the time the sentence shall be displayed (in miliseconds)
	 */
	public Sentence(AbstractCharacterSprite character, String line, int lineWidth, Color color, int time)
	{
		this.character = character;
		this.line = line;
		this.lineWidth = lineWidth;
		this.timer = new Timer(time);
		this.color = color;
		this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,20),Color.WHITE);
	}
	
	/**
	 * Creates a new sentence with the specified parameters
	 * @param character the character that says the sentence
	 * @param line the line to be spoken
	 * @param lineWidth width of a line until line wrap
	 * @param soundPath path+filename to sound file corresponding to
	 * this sentence
	 * @param color the color to display the sentence with
	 */
	public Sentence(AbstractCharacterSprite character, String line, int lineWidth, String soundPath, Color color)
	{
		this.character = character;
		this.line = line;
		this.lineWidth = lineWidth;
		this.soundPath = soundPath;
		this.color = color;
		this.screenWriter = new SystemFont(new Font("Arial",Font.BOLD,20),Color.WHITE);
	}
	
	/**
	 * Returns the character related to the sentence, i.e. the character that is
	 * speaking the sentence
	 * @return character related to the sentence
	 */
	public AbstractCharacterSprite getRelatedCharacter()
	{
		return this.character;
	}
	
	/**
	 * Returns the path+filename of the sound file that was
	 * assigned to this sentence
	 * @return sound file assigned to this sentence
	 */
	public String getAssignedSoundFile()
	{
		return this.soundPath;
	}
	
	/**
	 * Update method of the sentence
	 * @param elapsedTime time between two frames
	 */
	public void update(long elapsedTime)
	{
		if(this.timer != null && timer.action(elapsedTime))
		{
			this.sayNow = false;
			// stop the talking animation
			if(this.character.getAnimScheduler().getQueue().isEmpty() == false)
				this.character.getAnimScheduler().getQueue().removeFirst();
		}
		else
		{			
			if(this.character.getAnimScheduler().getQueue().isEmpty() == false
					&& (this.character.getAnimScheduler().getCurrAnimName() 
					!= "talk"))
			{
				// run the talking animation
				this.character.getAnimScheduler().clear();
				this.character.getAnimScheduler().add("talk");
			}
		}
		
		if(this.timer == null && this.soundPath != null)
		{	
			if(this.soundRunning == false)
			{				
				this.soundRunning = true;
				//this.character.getGameLocation().getSoundRenderer().stopAll();
				this.character.getGameLocation().playSound(this.soundPath);
			}
			else if(this.character.getGameLocation().getSoundRenderer().getAudioRenderer(this.soundPath).getStatus()
					== BaseAudioRenderer.END_OF_SOUND)
			{		
				this.character.getGameLocation().getSoundRenderer().getAudioRenderer(this.soundPath).stop();
				
				this.sayNow = false;
				// stop the talking animation
				if(this.character.getAnimScheduler().getQueue().isEmpty() == false)
					this.character.getAnimScheduler().getQueue().removeFirst();
			}
			
			/*
			if(this.character.getAnimScheduler().getQueue().isEmpty() == false
					&& (this.character.getAnimScheduler().getCurrAnimName() 
					!= "talk"))
			{
				// run the talking animation
				this.character.getAnimScheduler().clear();
				this.character.getAnimScheduler().add("talk");
			}
			*/
		}
	}
	
	/**
	 * Render method of the sentence. The sentence gets only rendered, if its
	 * display time has not expired
	 * @param g main render context
	 */
	public void render(Graphics2D g)
	{
		if(this.sayNow == true)
		{
			this.screenWriter.setColor(this.color);
			
			// calculate the number of pixels to shift the string to the left/right
			// in order to avoid exceeding the screen, if neccessary
			int x = this.calculatePosition(line).x;
			int y = this.calculatePosition(line).y;
			
			// set the color of the transparent background rectangle
			g.setColor(Color.BLACK);
			Composite compBackUp = g.getComposite();
			
			// set alpha to transparent
			g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, 0.55f));
			
			// set the width of the transparent rectangle around the sentence
			int rectWidth = 0;
			if(this.screenWriter.getWidth(this.line) > this.lineWidth)
			{
				rectWidth = this.lineWidth;
			}
			else rectWidth = this.screenWriter.getWidth(this.line);
			
			g.fillRect(x, y, rectWidth, this.heightPixels);
			
			this.screenWriter.drawText(g,line,GameFont.LEFT,x,y,this.lineWidth,0,0);						
			
			// set alpha to old value
			g.setComposite(compBackUp);
		}
	}
	
	/**
	 * Helper method to calculate the position of the sentence where to be
	 * displayed on screen.
	 * @param line the line to be said
	 * @return position where to draw the sentence
	 */
	private Point calculatePosition(String line)
	{
		int offsetY = 1*this.screenWriter.getHeight();
		int startX = (int)this.character.getX();
		int startY = (int)this.character.getY() -this.character.getHeight() - offsetY;			
		
		// let the sentence start shortly after the left edge if it 
		// gets clipped on the left screen edge
		if(startX < 0)	startX = 5;
		
		// shift the sentence to the left if it gets to long
		if(startX+this.lineWidth > 1020)
		{
			startX = (int)this.character.getX() -
			(((int)this.character.getX()+this.lineWidth) - 1020);
		}
		
		// find the number of lines the sentence gets wrapped to, to decide
		// whether the sentence must be drawn higher
		double numLines = Math.ceil((double)this.screenWriter.getWidth(line)/(double)this.lineWidth);
		
		// find the height in pixels the wrapped sentence will have
		this.heightPixels = (int)numLines * this.screenWriter.getHeight();
		
		// draw the sentence higher if it is neccessary 
		if(startY + heightPixels > startY-this.screenWriter.getHeight())
		{
			startY = startY - heightPixels;
		}
		
		return new Point(startX, startY);
	}
	
	/**
	 * Determines whether the sentence is currently spoken by a character
	 * @return is sentence currently spoken? --> true=yes, false=no
	 */	
	public boolean isCurrentlySpoken()
	{
		return this.sayNow;
	}
}
