package xml;

import gameobjects.characters.NPCSprite;
import gameobjects.characters.Player;
import gameobjects.item.Item;
import gameobjects.prop.Prop;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import location.GameLocation;
import location.InteractiveHotspot;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;

/**
 * This class represents a loaction loader that can be used to load
 * game location specifications from xml files
 * @author Frank Bruns
 *
 */
public class LocationLoader
{
	private BaseLoader bsLoader = null;
	private BaseInput bsInput = null;
	private BaseAudio bsSound = null;
	private BaseAudio bsMusic = null;
	private Document locDoc = null;
	private Document charDoc = null;
	private Document itemDoc = null;
	
	public LocationLoader(BaseLoader bsLoader,BaseInput bsInput,BaseAudio bsSound,BaseAudio bsMusic)
	{
		this.bsLoader = bsLoader;
		this.bsInput = bsInput;
		this.bsSound = bsSound;
		this.bsMusic = bsMusic;
	}
	
	/**
	 * Helper method to parse the xml file specified by the file name
	 * @param fileName path+file name of the xml file 
	 * @return a document representation of the xml file
	 * @throws DocumentException
	 */
	private Document parseXML(String fileName) throws DocumentException {
        SAXReader reader = new SAXReader();
        Document document = reader.read(fileName);

        return document;
    }
	
	/**
	 * Returns the list of game locations specified by the xml files
	 * from the file names array
	 * @param xmlFiles xml files that specify the game locations
	 * @return list of game locations
	 */
	public ArrayList<GameLocation> getLocationFromXML(List<File> xmlFiles)
	{
		ArrayList<GameLocation> locations = new ArrayList<GameLocation>();
		
		for(File xmlFile : xmlFiles)
		{			
			try
			{
				this.locDoc = this.parseXML(xmlFile.getPath());
			} 
			catch (DocumentException e)
			{
				System.out.println("Error while parsing xml file");
				e.printStackTrace();
			}
			
			// read game loction infos
			String name = locDoc.selectSingleNode("//location/name").getText();
			String bshName = locDoc.selectSingleNode("//location/bshname").getText();
			String bshScript = locDoc.selectSingleNode("//location/script").getText();
			String bgImage = locDoc.selectSingleNode("//location/bgimage").getText();
			String imgWalkable = locDoc.selectSingleNode("//location/bgwalkable").getText();
			String bgMusic = locDoc.selectSingleNode("//location/bgmusic").getText();
			int scale = locDoc.numberValueOf("//location/scale").intValue();
			GameLocation gameLocation = new GameLocation(name,bshName,xmlFile.getPath(),bshScript,bgImage,imgWalkable,bgMusic,scale,bsLoader,bsInput,bsSound,bsMusic);
			
			// read character infos
			this.addCharacters(gameLocation);						
			
			// read item infos
			this.addItems(gameLocation);
						
			// read interactive hotspot infos
			this.addInteractiveHotspots(gameLocation);
			
			// read prop infos
			this.addProps(gameLocation);
			
			// initialize the pathfinding search grid
			//gameLocation.initSearchGrid();
			
			locations.add(gameLocation);
		}
		
		
		return locations;
	}
	
	/**
	 * Helper method to read the character information from the xml document.
	 * @param gameLocation game location object to add the characters to
	 */
	private void addCharacters(GameLocation gameLocation)
	{		
		List<Node> npcs = this.locDoc.selectNodes("//location/character");
		for(Node npcNode : npcs)
		{
			try
			{
				this.charDoc = this.parseXML(npcNode.valueOf("file"));
			}
			catch (DocumentException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			Node characterNode = this.charDoc.selectSingleNode("//character");
			String name = characterNode.valueOf("name");
			String bshName = characterNode.valueOf("bshname");			
			double x = npcNode.numberValueOf("x").doubleValue();
			double y = npcNode.numberValueOf("y").doubleValue();
			Color color = new Color(
					characterNode.numberValueOf("color/red").intValue(),
					characterNode.numberValueOf("color/green").intValue(),
					characterNode.numberValueOf("color/blue").intValue()
					);
			NPCSprite character = new NPCSprite(name,bshName,x,y,color,bsLoader,bsInput,bsSound);
			
			// read animation infos
			List<Node> animations = characterNode.selectNodes("animation");
			for(Node anim : animations)
			{
				name = anim.valueOf("name");

				BufferedImage[] images = bsLoader.getImages(
						anim.valueOf("img"),
						anim.numberValueOf("img/@picsperrow").intValue(),
						anim.numberValueOf("img/@rows").intValue());
				character.getAnimPool().add(name,images);
			}
			
			character.getAnimScheduler().start();
			gameLocation.addCharacter(character);
		}		
	}
	
	/**
	 * Helper method to read the item information from the xml document.
	 * @param gameLocation game location object to add the items to
	 */
	private void addItems(GameLocation gameLocation)
	{
		List<Node> items = locDoc.selectNodes("//location/item");
		for(Node itemNodeLoc : items)
		{
			try
			{
				this.itemDoc = this.parseXML(itemNodeLoc.valueOf("file"));
			}
			catch (DocumentException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			Node itemNode = this.itemDoc.selectSingleNode("//item");
			String name = itemNode.valueOf("name");
			String bshName = itemNode.valueOf("bshname");				
			double x = itemNodeLoc.numberValueOf("x").doubleValue();
			double y = itemNodeLoc.numberValueOf("y").doubleValue();
			boolean pickable = new Boolean(itemNode.valueOf("pickable"));			
			boolean visible = new Boolean(itemNode.valueOf("visible"));
			BufferedImage inventImage = bsLoader.getImage(itemNode.valueOf("invimg"));
			
			Item item = new Item(name,bshName,x,y,pickable,visible,inventImage,bsLoader,bsInput,bsSound);
			
			// read animation infos
			List<Node> animations = itemNode.selectNodes("animation");
			for(Node anim : animations)
			{
				name = anim.valueOf("name");

				BufferedImage[] images = bsLoader.getImages(
						anim.valueOf("img"),
						anim.numberValueOf("img/@picsperrow").intValue(),
						anim.numberValueOf("img/@rows").intValue());
				item.getAnimPool().add(name,images);
			}
			
			item.getAnimScheduler().start();
			gameLocation.addItem(item);
		}		
	}
	
	/**
	 * Helper method to read the interactive hotspot information 
	 * from the xml document.
	 * @param gameLocation game location object to add the interactive
	 * hotspot to
	 */
	private void addInteractiveHotspots(GameLocation gameLocation)
	{
		List<Node> hotspots = locDoc.selectNodes("//location/hotspot");
		for(Node hsNode : hotspots)
		{
			String name = hsNode.valueOf("name");
			String bshName = hsNode.valueOf("bshname");				
			double x = hsNode.numberValueOf("x").doubleValue();
			double y = hsNode.numberValueOf("y").doubleValue();
			int width = hsNode.numberValueOf("width").intValue();
			int height = hsNode.numberValueOf("height").intValue();
			
			InteractiveHotspot ihs = new InteractiveHotspot(name,bshName,x,y,width,height,bsInput,bsLoader);
			gameLocation.addInteractiveHotspot(ihs);
		}		
	}
	
	/**
	 * Helper method to read the prop information from the xml document.
	 * @param gameLocation game location object to add the prop to
	 */
	private void addProps(GameLocation gameLocation)
	{
		List<Node> items = locDoc.selectNodes("//location/prop");
		for(Node itemNode : items)
		{			
			String bshName = itemNode.valueOf("bshname");				
			double x = itemNode.numberValueOf("x").doubleValue();
			double y = itemNode.numberValueOf("y").doubleValue();					
			
			Prop prop = new Prop(bshName,x,y,bsLoader,bsInput,bsSound);
			
			// read animation infos
			List<Node> animations = itemNode.selectNodes("animation");
			for(Node anim : animations)
			{
				String name = anim.valueOf("name");

				BufferedImage[] images = bsLoader.getImages(
						anim.valueOf("img"),
						anim.numberValueOf("img/@picsperrow").intValue(),
						anim.numberValueOf("img/@rows").intValue());
				prop.getAnimPool().add(name,images);
			}
			
			prop.getAnimScheduler().start();
			gameLocation.addProp(prop);
		}		
	}
	
	/**
	 * Method to add the locatios exits to the list of game locations.
	 * The information will be retrieved from the xml files
	 * <br>
	 * <b>Note: </b>Call this method after all game locations have
	 * been initialized.
	 * @param locations list of existing game locations
	 */
	public ArrayList<GameLocation> getLocationsWithAddedExits(ArrayList<GameLocation> locations)
	{		
		for(GameLocation gl : locations)
		{
			// read the xml file for the game location to be able to
			// retrieve its information about exits
			Document doc = null;
			try
			{
				doc = this.parseXML(gl.getXMLFileName());
			} catch (DocumentException e)
			{
				System.out.println("Error while parsing xml file");
				e.printStackTrace();
			}
			
			List<Node> exits = doc.selectNodes("//location/exit");
			for(Node exitNode : exits)
			{
				String name = exitNode.valueOf("name");
				String bshName = exitNode.valueOf("bshname");				
				double x = exitNode.numberValueOf("x").doubleValue();
				double y = exitNode.numberValueOf("y").doubleValue();
				int width = exitNode.numberValueOf("width").intValue();
				int height = exitNode.numberValueOf("height").intValue();
				double newX = exitNode.numberValueOf("newx").doubleValue();
				double newY = exitNode.numberValueOf("newy").doubleValue();
				boolean usable = new Boolean(exitNode.valueOf("usable"));
				String destination = exitNode.valueOf("destination");	
				
				GameLocation destLocation = this.findGameLocation(locations, destination);
				if(destLocation != null)
				{
					gl.addExit(name, bshName, x, y, width, height, newX, newY, usable, destLocation);
				}
			}
		}					
		
		return locations;
	}
	
	/**
	 * Method to add the player to the list of game locations.
	 * The information will be retrieved from the player.xml file
	 * <br>
	 * <b>Note: </b>Call this method after all game locations have
	 * been initialized.
	 * @param locations list of existing game locations
	 */
	public ArrayList<GameLocation> getLocationsWithAddedPlayer(ArrayList<GameLocation> locations)
	{				
		// read the player.xml file to be able to
		// retrieve its information about the player object
		Document doc = null;
		try
		{
			doc = this.parseXML("./player/player.xml");
		} catch (DocumentException e)
		{
			System.out.println("Error while parsing xml file");
			e.printStackTrace();
		}
		
		Node playerNode = doc.selectSingleNode("//player");
		
		String name = playerNode.valueOf("name");
		String bshName = playerNode.valueOf("bshname");				
		String startLocName = playerNode.valueOf("startlocation");
		double x = playerNode.numberValueOf("x").doubleValue();
		double y = playerNode.numberValueOf("y").doubleValue();
		int red = playerNode.numberValueOf("color/red").intValue();
		int green = playerNode.numberValueOf("color/green").intValue();
		int blue = playerNode.numberValueOf("color/blue").intValue();							
	
		// create player object
		Player player = new Player(name,bshName,x,y,new Color(red,green,blue),bsLoader,bsInput,bsSound);
		
		
		// read animation infos
		List<Node> animations = playerNode.selectNodes("animation");
		for(Node anim : animations)
		{
			name = anim.valueOf("name");

			BufferedImage[] images = bsLoader.getImages(
					anim.valueOf("img"),
					anim.numberValueOf("img/@picsperrow").intValue(),
					anim.numberValueOf("img/@rows").intValue());
			player.getAnimPool().add(name,images);
		}
		
		player.getAnimScheduler().start();
		this.findGameLocation(locations, startLocName).addPlayer(player);
		
		return locations;
	}
	
	/**
	 * Helper method to retrieve the game location with the specified name 
	 * from the list of game locations
	 * @param locations locations to search in
	 * @param name name of the game location to be found
	 * @return found game location, null if none was found
	 */
	private GameLocation findGameLocation(ArrayList<GameLocation> locations,String name)
	{
		for(GameLocation g : locations)
		{
			if(g.getName().equals(name)) return g;
		}
		return null;
	}
}
