package gameaction;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;

import speech.Sentence;
import speech.SentenceScheduler;
import state.GameState;
import gameobjects.characters.AbstractCharacterSprite;
import location.GameLocation;

/**
 * A game action that represents talking to a character
 * @author Frank Bruns
 *
 */
public class Talk extends AbstractGameAction
{
	private SentenceScheduler sentenceScheduler = new SentenceScheduler();
	ArrayList<Sentence> sentencesBackup = null;
	private static final int LINE_WIDTH = 400;
	private boolean isBranch = false;
	private boolean isAborter = false;
	
	/**
	 * Creates a new Talk action
	 * @param c character to whose game action queue the talk
	 * action shall be assigned to
	 * @param isBranch specifies, if the talk action is part of
	 * a dialog with a question selection area
	 */
	public Talk(AbstractCharacterSprite c, boolean isBranch)
	{
		super(c);
		this.isBranch = isBranch;
		this.sentencesBackup = new ArrayList<Sentence>();
	}
	
	/**
	 * Creates a new Talk action
	 * @param c character to whose game action queue the talk
	 * action shall be assigned to
	 * @param isBranch specifies, if the talk action is part of
	 * a dialog with a question selection area
	 * @param aborter specifies if the talk action is used to
	 * abort dialogs with question selection areas
	 */
	public Talk(AbstractCharacterSprite c, boolean isBranch, boolean aborter)
	{
		super(c);
		this.isBranch = isBranch;
		this.isAborter = aborter;
		this.sentencesBackup = new ArrayList<Sentence>();
	}
	
	/**
	 * Determines whether this talk actions is the aborter
	 * for a dialog
	 * @return is aborter? --> true=yes, false=no
	 */
	public boolean isAborter()
	{
		return this.isAborter;
	}
	
	/**
	 * Returns the global sentence scheduler
	 * @return the sentence scheduler
	 */
	public SentenceScheduler getSentenceScheduler()
	{
		return sentenceScheduler;
	}
	
	/**
	 * Determines whether there are no more sentences in the 
	 * sentences scheduler
	 * @return nothing more to say? --> true=yes, false=no
	 */
	public boolean isNoMoreToSay()
	{
		if(this.sentenceScheduler.getSentences().isEmpty())
		{
			// refill the sentence scheduler for later re-call
			// of this game action in a possible dialog
			for(Sentence s : this.sentencesBackup)
			{
				sentenceScheduler.add(s);
			}
			return true;
		}
		else return false;
	}
	
	/**
	 * Lets the character say a sentence with the specified color and
	 * specified sound file
	 * @param c the character that says something
	 * @param line sentence to be said
	 * @param color color of the sentence's text
	 * @param soundPath path+filename for corresponding sound file
	 */
	public void say(AbstractCharacterSprite c, String line, String soundPath, Color color)
	{
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,soundPath,color));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,soundPath,color));
	}
	
	/**
	 * Lets the character say a sentence with the specified color
	 * @param c the character that says something
	 * @param line sentence to be said
	 * @param color color of the sentence's text
	 * @param time time in ms the line is visible
	 */
	public void say(AbstractCharacterSprite c, String line, Color color, int time)
	{
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,color,time));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,color,time));
	}
	

	/**
	 * Lets the character say a sentence with the specified sound file
	 * @param c character that says something
	 * @param line sentence to be said
	 * @param soundPath path+filename for corresponding sound file
	 */
	public void say(AbstractCharacterSprite c,String line, String soundPath)
	{
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,soundPath,c.getSpeechColor()));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,soundPath,c.getSpeechColor()));
	}
	
	/**
	 * Lets the character say a sentence
	 * @param c character that says something
	 * @param line sentence to be said
	 * @param time time in ms the line is visible
	 */
	public void say(AbstractCharacterSprite c,String line, int time)
	{
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,c.getSpeechColor(),time));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,c.getSpeechColor(),time));
	}
	
	/**
	 * Lets the character say a sentence. The duration time for displaying
	 * the sentence varies by the number of characters in the sentence.
	 * A longer sentence logically means a longer time of display
	 * @param c character that says something
	 * @param line sentence to be said
	 */
	public void say(AbstractCharacterSprite c, String line)
	{
		// The time for displaying the sentence is calculated by assuming 
		// that every letter needs approx. 100 miliseconds to be read
		int time = line.length() * 100; 
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,c.getSpeechColor(),time));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,c.getSpeechColor(),time));
	}
	
	/**
	 * Lets the character say a sentence.
	 * <br>
	 * <b>Note: </b>This method cannot be called directly from 
	 * beanshell scripts
	 * @param c character that says something
	 * @param lines one or several sentences to be said
	 * @deprecated do not use this method anymore
	 */
	public void say(AbstractCharacterSprite c,String[] lines)
	{
		for(int i=0; i<lines.length; i++) this.say(c,lines[i]);
	}
	
	/**
	 * Lets the character say a sentence with the specified color.
	 * The duration time for displaying the sentence varies by the 
	 * number of characters in the sentence. A longer sentence 
	 * logically means a longer time of display.
	 * @param c character that says something
	 * @param line sentence to be said
	 * @param color color of the sentence's text
	 */
	public void say(AbstractCharacterSprite c, String line, Color color)
	{
		// The time for displaying the sentence is calculated by assuming 
		// that every letter needs approx. 100 miliseconds to be read
		int time = line.length() * 100; 
		sentenceScheduler.add(new Sentence(c,line,LINE_WIDTH,color,time));
		this.sentencesBackup.add(new Sentence(c,line,LINE_WIDTH,color,time));
	}
	
	@Override
	public void update(long elapsedTime)
	{		
		if(this.secondMethodCall == true)
		{
			if(!GameState.equals(GameState.DIALOG) 
					&& !GameState.equals(GameState.CUTSCENE))
			{
				GameState.setState(GameState.DIALOG);
			}
			
			sentenceScheduler.update(elapsedTime);
			if(this.isNoMoreToSay() == true) 
			{
				this.character.getGameActionScheduler().removeFirst();	
				
				if(GameState.equals(GameState.DIALOG))
				{
					if(this.isBranch == true)GameState.setState(GameState.DIALOG_SELECTION);
					if(this.isBranch == true && this.isAborter == true)GameState.setState(GameState.NORMAL);
					if(this.isBranch == false)GameState.setState(GameState.NORMAL);
				}
			}
		}
		else this.secondMethodCall = true;
	}
	
	public void render(Graphics2D g)
	{
		sentenceScheduler.render(g);		
	}
}
