package gameaction;

import java.awt.Point;

import location.InteractiveHotspot;
import path.Path;
import state.GameState;

import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.NPCSprite;
import gameobjects.characters.Player;
import gameobjects.item.Item;

/**
 * A game action that represents walking to a new point
 * in the game location
 * @author Frank Bruns
 *
 */
public class Walk extends AbstractGameAction
{
	private Path pathToWalk = null;
	private double targX = 0;
	private double targY = 0;
	
	private boolean hasArrivedAtTarget = false;
	
	/**
	 * Creates a new walking event with the specified coordinates
	 * that provide the target location of the walking event
	 * @param c the character that walks
	 * @param goToX target x-coordinate for the walking event
	 * @param goToY target y-coordinate for the walking event
	 */
	public Walk(AbstractCharacterSprite c,int goToX, int goToY)
	{
		super(c);
		//Point targetPos = this.checkGoNextToCharacterOrItem(goToX,goToY);
		this.targX = goToX;
		this.targY = goToY;		
		if(c instanceof Player)
		{
			Player player = (Player)c;
			player.setHasArrivedAtTarget(false);
		}
	}
	
	/**
	 * Creates a new walking event with the specified coordinates
	 * that provide the target location of the walking event
	 * @param c the character that walks
	 * @param path path to walk for the character
	 */
	public Walk(AbstractCharacterSprite c, Path path)
	{
		super(c);

		this.pathToWalk = path;
		
		if(this.pathToWalk.getPoints().isEmpty() == false)
		{
			this.targX = this.pathToWalk.getPoints().getLast().x;
			this.targY = this.pathToWalk.getPoints().getLast().y;
		}
		else
		{
			this.targX = this.character.getX();
			this.targY = this.character.getY();
		}
		
		if(c instanceof Player)
		{
			Player player = (Player)c;
			player.setHasArrivedAtTarget(false);
		}
	}
	
	/**
	 * Returns the x-coordinate of the walking target position
	 * @return target x-coordinate
	 */
	public double getTargetX()
	{
		return this.targX;
	}
	
	/**
	 * Returns the y-coordinate of the walking target position
	 * @return target y-coordinate
	 */
	public double getTargetY()
	{
		return this.targY;
	}
	
	/**
	 * Returns whether the character has arrived at its designated target
	 * location.
	 * @return has arrived at target location? --> true=yes, false=no
	 */
	public boolean hasArrivedAtTarget()
	{
		return this.hasArrivedAtTarget;
	}
	
	/**
	 * This helper method is used to prevent the player character
	 * from standing right on another character. If the new target position
	 * of the player would be on another character, the target position will
	 * be changed to right in front of the other character
	 * @param mouseX current mouse x-coordinate
	 * @param mouseY current mouse y-coordinate
	 * @deprecated this method is not used any more since 
	 * the introduction of A-Star path finding
	 */
	private Point checkGoNextToCharacterOrItem(int mouseX, int mouseY)
	{
		Point point = new Point(mouseX,mouseY);		
		
		for(AbstractCharacterSprite c: this.character.getGameLocation().getNonPlayerCharacters())
		{
			if(c.getHotspot().isPointInHotspot(mouseX, mouseY))
			{
				point.x = (int)c.getX()-this.character.getWidth();
				point.y = (int)c.getY();
				return point;
			}
		}
		
		for(Item i : this.character.getGameLocation().getItems())
		{
			if(i.getHotspot().isPointInHotspot(mouseX, mouseY))
			{
				point.x = (int)i.getX()-this.character.getWidth();
				point.y = (int)i.getY()-this.character.getHeight()+i.getHeight();
				return point;
			}
		}
		
		for(InteractiveHotspot hs : this.character.getGameLocation().getInteractiveHotspots())
		{
			if(hs.isPointInHotspot(mouseX, mouseY))
			{
				point.x = (int)hs.getX()-this.character.getWidth();
				point.y = (int)hs.getY()-this.character.getHeight()+hs.getHeight();
				return point;
			}
		}
		
		point.x = mouseX;
		point.y = mouseY;
		return point;
	}

	@Override
	public void update(long elapsedTime)
	{		
		// Don't let the player do anything if an NPC is walking
		if(this.character instanceof NPCSprite)
		{
			if(GameState.equals(GameState.NO_INPUT_PROCESSING) == false
					&& GameState.equals(GameState.CUTSCENE) == false)
			GameState.setState(GameState.NO_INPUT_PROCESSING);
		}
		
		if(this.character.getAnimScheduler().getQueue().isEmpty() == false)
		{						
			if(this.character.getAnimScheduler().getCurrAnimName() == "standard")				
			{
				this.character.getAnimScheduler().getQueue().removeFirst();
			}
		}
		
							
		if(this.pathToWalk == null) // path not set, other constructor was used
		{	
			if(this.character.getAnimScheduler().getQueue().isEmpty())
			{
				if(this.character.getX() > this.getTargetX())
				{
					this.character.setDirection(AbstractCharacterSprite.DIR_LEFT);
					this.character.getAnimScheduler().add("walk_left");
				}
				if(this.character.getX() < this.getTargetX())
				{
					this.character.setDirection(AbstractCharacterSprite.DIR_RIGHT);
					this.character.getAnimScheduler().add("walk_right");
				}
			}
			
			this.character.moveTo(elapsedTime,this.getTargetX(),this.getTargetY(), 0.25);
		}
		// process this branch if a path object was passed via constructor
		else if(this.pathToWalk.getCurrentPoint() != null)
		{
			switch(this.character.getDirection())
			{
				case AbstractCharacterSprite.DIR_NO_DIR : 
					{
						if(this.character.getX() > this.pathToWalk.getCurrentPoint().x)
						{
							this.character.setDirection(AbstractCharacterSprite.DIR_LEFT);
							this.character.getAnimScheduler().clear();
							this.character.getAnimScheduler().add("walk_left");					
						}
						else
						{
							this.character.setDirection(AbstractCharacterSprite.DIR_RIGHT);
							this.character.getAnimScheduler().clear();
							this.character.getAnimScheduler().add("walk_right");
						}
						break;
					}
				case AbstractCharacterSprite.DIR_LEFT : 
					{
						if(this.character.getX() < this.pathToWalk.getCurrentPoint().x)
						{
							this.character.setDirection(AbstractCharacterSprite.DIR_RIGHT);
							this.character.getAnimScheduler().clear();
							this.character.getAnimScheduler().add("walk_right");					
						}
						break;
					}
				case AbstractCharacterSprite.DIR_RIGHT : 
					{
						if(this.character.getX() > this.pathToWalk.getCurrentPoint().x)
						{
							this.character.setDirection(AbstractCharacterSprite.DIR_LEFT);
							this.character.getAnimScheduler().clear();
							this.character.getAnimScheduler().add("walk_left");					
						}
						break;
					}
			}
		
			
			this.character.moveTo(elapsedTime, 
					this.pathToWalk.getCurrentPoint().x,
					this.pathToWalk.getCurrentPoint().y, 0.25);
		}
		
		//System.out.println("CharX: "+this.character.getX() +"  ,CharY:"+this.character.getY());
		//System.out.println("TargX: "+this.getTargetX() +"  ,TargY:"+this.character.getTargetY());
		
		if(this.character.getX() == this.getTargetX() 
				&& this.character.getY() == this.getTargetY())
		{
			this.hasArrivedAtTarget = true;
			this.character.setDirection(AbstractCharacterSprite.DIR_NO_DIR);
			if(this.character instanceof Player)
			{
				Player player = (Player)this.character;
				player.setHasArrivedAtTarget(true);
			}
			
			
			if(this.hasArrivedAtTarget())
			{	
				if(this.getCharacter().getAnimScheduler().getQueue().isEmpty() == false)
				this.getCharacter().getAnimScheduler().getQueue().removeFirst();
				
				if(this.getCharacter().getGameActionScheduler().getGameActions().isEmpty() == false)
				this.getCharacter().getGameActionScheduler().removeFirst();
				
				// Return to normal state after an NPC arrived at destination
				if(this.character instanceof NPCSprite)
				{
					if(GameState.equals(GameState.NORMAL) == false
							&& GameState.equals(GameState.CUTSCENE) == false)
					GameState.setState(GameState.NORMAL);
				}
			}
		}
		
		//System.out.println("CharX: "+this.character.getX() +"  ,CharY:"+this.character.getY());
		//System.out.println("PathX: "+this.pathToWalk.getCurrentPoint().x +"  ,PathY:"+this.pathToWalk.getCurrentPoint().y);
		if(this.pathToWalk != null 
				&& this.pathToWalk.getPoints().isEmpty() == false)
		{
			if(this.character.getX() == (double)this.pathToWalk.getCurrentPoint().x
					&& this.character.getY() == (double)this.pathToWalk.getCurrentPoint().y)
			{
				this.pathToWalk.next();
				//System.out.println("next() was called");
			}
		}
	}
}
