package gameobjects.characters;

import gameobjects.item.Item;

import inventory.Inventory;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.HashMap;

import scripting.BeanShell;
import speech.Dialog;
import speech.QuestionSelectionArea;
import state.GameState;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;

/**
 * This class represents the main character sprite of the game. This character
 * is controlled by the player.
 * @author Frank Bruns
 *
 */
public class Player extends AbstractCharacterSprite
{	
	/**
	 * Constant that refers to the standard animation of a game object
	 */
//	public static final String STANDARD = "standard";	
//	/** constant refering to the waiting animation */
//	public static final String WAIT = "wait";
//	/** constant refering to the walking right animation */	
//	public static final String WALK_RIGHT = "walk_right";
//	/** constant refering to the walking left animation */
//	public static final String WALK_LEFT = "walk_left";	
//	/** constant refering to the talking animation */
//	public static final String TALK = "talk";
//	public static final String DIGGING = "digging";
	
	private Inventory inventory = null;	
	private QuestionSelectionArea qsa = null;
	//private HashMap<String,CutScene> cutsceneSet = null;
	
	private boolean hasArrivedAtTarget = true;
	private boolean isInDialog = false;
	
	/**
	 * Creates a new player sprite.
	 * @param name Name of the player character
	 * @param bshName name that will be used for the player in
	 * beanshell scripts. I recommend to simply use "player" as the 
	 * beanshell name
	 * @param x x-coordinate, where the sprite should initially appear
	 * @param y y-coordinate, where the sprite should initially appear
	 * @param speachColor color for the spoken dialog lines on screen
	 * @param bsLoader the BaseLoader object from the Main class
	 * @param bsSound the BaseAudio object from the Main class
	 * @param bsInput the BaseInput object from the Main class
	 */
	public Player(String name, String bshName, double x, double y, Color speachColor, BaseLoader bsLoader, BaseInput bsInput, BaseAudio bsSound)
	{		
		super(name, bshName, x, y, speachColor, bsLoader, bsInput, bsSound);
		this.inventory = new Inventory(1024,50,bsInput);
		this.qsa = new QuestionSelectionArea(this,bsInput);				
	}
	
	
	/**
	 * Returns whether the player has arrived at his designated target
	 * location.
	 * @return has arrived at target location? --> true=yes, false=no
	 */
	public boolean hasArrivedAtTarget()
	{
		return this.hasArrivedAtTarget;
	}
	
	/**
	 * Sets whether the player has arrived at target
	 * @param arrived has arrived? --> true=yes, false=no
	 */
	public void setHasArrivedAtTarget(boolean arrived)
	{
		this.hasArrivedAtTarget = arrived;
	}
	
	/**
	 * Returns the player's inventory containing the collected items
	 * @return the player's inventory
	 */
	public Inventory getInventory()
	{
		return this.inventory;
	}
	
	/**
	 * Convenience method to determine whether the player is currently
	 * looking at the inventory
	 * @return is player looking at inventory? --> true=yes, false=no
	 */
	public boolean isLookingAtInventory()
	{
		return this.inventory.isLookingAtInventory();
	}
	
	/**
	 * Adds a cutscene to the set of cutscenes that are 
	 * accessible by the player object
	 * @param name indentifying name for the cutscene
	 * @param scriptFile beanshell scriptfile defining the cutscene
	 */
	/*
	public void addCutScene(String name, String scriptFile)
	{
		this.cutsceneSet.put(name, new CutScene(scriptFile));
	}
	*/
	
	/**
	 * Helper method to run a scripted cutscene
	 * @param scriptFile beanshell script file for the cutscene
	 */
	/*
	public void runCutScene(String name)
	{
		if(this.cutsceneSet.get(name).isAllowdToRun() == true)
		BeanShell.executeScript(this.cutsceneSet.get(name).getScriptFile());
	}
	*/
	
	/**
	 * Determines whether the player is currently making conversation 
	 * @return is player making conversation? --> true=yes, false=no
	 */
	public boolean isInDialog()
	{
		return this.isInDialog;
	}
	
	/**
	 * Sets the dialog current status of the player
	 * @param isInDialog is making conversation? --> true=yes, false=no
	 */
	public void setIsInDialog(boolean isInDialog)
	{
		this.isInDialog = isInDialog;
	}
	
	/**
	 * Convenience method to run the dialog for the player 
	 * @param dialogName indentifying name of the dialog to 
	 * apply with the player
	 */
	public void runDialog(String dialogName)
	{
		this.qsa.setDialog(dialogName);
	}
	
	/**
	 * Adds a dialog to the set of player dialogs
	 * @param dialog the dialog to add
	 */
	public void addDialog(Dialog dialog)
	{
		this.qsa.addDialog(dialog);
	}
	
	public Dialog getDialog(String name)
	{
		return this.qsa.getDialog(name);
	}
	
	/**
	 * Convenience method for simpler use in beanshell scripts to
	 * get the player's currently selected inventory item. This method
	 * returns null, if currently none is selected
	 * @return player's currently selected inventory item, null if none is
	 * seleceted
	 */
	public Item getInventItem()
	{
		return this.inventory.getSelectedInventoryItem();
	}
	
	/**
	 * This method checks, if the currently selected inventory item
	 * is equal to the specified item. 
	 * @param item item to check for use
	 * @return does the player currently use the specified item? --> true=yes, false=no
	 */
	public boolean uses(Item item)
	{
		if(this.getInventItem() != null &&
				this.getInventItem().equals(item) == true) return true;
		else return false;
	}
	
	/**
	 * Convenience method to check if the specified item is contained in the
	 * player's inventory
	 * @param item item to check for existence in the player's inventory
	 * @return inventory contains the item? --> true=yes, false=no
	 */
	public boolean hasItem(Item item)
	{
		return this.inventory.hasItem(item);
	}
	
	/**
	 * Takes an item and adds it to the player's inventory while 
	 * removing it from the game location it was taken from
	 * @param item item to pick and add to the player's inventory
	 */
	public void pickItem(Item item)
	{
		this.inventory.addItemToInventory(item);
		item.setToPicked();
		this.getGameLocation().removeItem(item);		
	}

	/**
	 * Convenience method to remove an item from the player's inventory
	 * @param item item to remove from inventory
	 */
	public void loseItem(Item item)
	{
		this.inventory.removeItem(item);
	}
	
	@Override
	public void update(long elapsedTime)
	{
		super.update(elapsedTime);		
		this.inventory.update(elapsedTime);
		this.qsa.update(elapsedTime);
	}
	
	@Override
	public void render(Graphics2D g)
	{		
		super.render(g);
		this.inventory.render(g);
		this.qsa.render(g);
	}
}
