package location;

import gameaction.AbstractGameAction;
import gameaction.Leave;
import gameaction.Talk;
import gameaction.Walk;
import gameobjects.AbstractGameObject;
import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.NPCSprite;
import gameobjects.characters.Player;
import gameobjects.item.Item;
import gameobjects.prop.Prop;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import path.AStarSearch;
import path.GridNode;
import path.PathFinder;
import path.SearchGrid;

import scripting.BeanShell;
import state.GameState;

import com.golden.gamedev.engine.BaseAudio;
import com.golden.gamedev.engine.BaseAudioRenderer;
import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.BaseLoader;
import com.golden.gamedev.object.Background;
import com.golden.gamedev.object.background.ImageBackground;
import com.golden.gamedev.util.ImageUtil;

import cursor.CursorManager;
import cursor.Cursors;

/**
 * This class reprsents a location with all its contents.
 * @author Frank Bruns
 *
 */
public class GameLocation implements Serializable
{
	private String xmlFile = "no file name yet";
	private String bshScriptFile = "no file name yet";
	private BufferedImage imageWalkable = null;
	private String bgMusicFile = "no background music file yet";
	private Player player = null;
	private String name = "no name yet";
	private String bshName = "no beanshell name yet";
	private float scaleFactor = 1;
	
	private BaseLoader bsLoader = null;
	private BaseInput bsInput = null;
	private BaseAudio bsSound = null;
	private BaseAudio bsMusic = null;
	
	private Background background = null;
	private ArrayList<NPCSprite> characters = null;
	private ArrayList<AbstractGameObject> removeGameObjects = null;
	private ArrayList<Item> items = null;
	private ArrayList<InteractiveHotspot> interactiveHotspots = null;
	private ArrayList<Prop> props = null;
	private LinkedList<AbstractGameObject> sortedRenderList = null;
	
	private ArrayList<LocationExit> exits = null;
	
	private SearchGrid pathSearchGrid = null;
	
	private boolean isActive = false;
	
	// for path debugging
	private List<GridNode> pathNodes = null;
	private boolean visGrid = false;
	private boolean stopRendering = false;
	
	private BufferedImage intermissionScreen = null;
	
	private int counter = 99;
	
	
	
	/**
	 * Creates a new game location with the specified parameters
	 * @param name name of the game location
	 * @param bshName beanshell name for useage in beanshell script files
	 * @param xmlFile path+file name of the assigned xml file
	 * @param bshScriptFile path+filename to beanshell script file for
	 * this game location. Call setUpBshScript() after the game loactions has
	 * been fully instantiated with characters and items
	 * @param pathBGImg path to the background image for the loaction
	 * @param bgMusicFile path+file name to music file
	 * @param scale sacling factor for the characters in the location
	 * @param bsLoader
	 * @param bsInput
	 * @param bsSound
	 */
	public GameLocation(String name, String bshName, String xmlFile, 
			String bshScriptFile, String pathBGImg, String imgWalkable,
			String bgMusicFile, int scale, BaseLoader bsLoader, BaseInput bsInput, 
			BaseAudio bsSound, BaseAudio bsMusic)
	{
		this.bsLoader = bsLoader;
		this.bsInput = bsInput;
		this.bsSound = bsSound;		
		this.bsMusic = bsMusic;
		
		this.name = name;
		this.bshName = bshName;
		this.xmlFile = xmlFile;
		this.bshScriptFile = bshScriptFile;
		this.background = new ImageBackground(bsLoader.getImage(pathBGImg));
		this.imageWalkable = bsLoader.getImage(imgWalkable);
		this.bgMusicFile = bgMusicFile;
		this.scaleFactor = (100-scale) / 767f;
		
		this.sortedRenderList = new LinkedList<AbstractGameObject>();
		this.characters = new ArrayList<NPCSprite>();
		this.removeGameObjects = new ArrayList<AbstractGameObject>();
		this.items = new ArrayList<Item>();
		this.interactiveHotspots = new ArrayList<InteractiveHotspot>();
		this.props = new ArrayList<Prop>();
		this.exits = new ArrayList<LocationExit>();
				
		this.pathNodes = new ArrayList<GridNode>();
	}
	
	/**
	 * Returns the path+filename of the xml  file assigned 
	 * to this game location
	 * @return path+filename of the xml file
	 */
	public String getXMLFileName()
	{
		return this.xmlFile;
	}
	
	/**
	 * Returns the path+filename of the beanshell script file assigned 
	 * to this game location
	 * @return path+filename of the beanshell script file
	 */
	public String getScriptFileName()
	{
		return this.bshScriptFile;
	}
	
	/**
	 * Returns the path+file name of the background music theme
	 * of this game location
	 * @return path+file name of the background music theme
	 */
	public String getMusicFileName()
	{
		return this.bgMusicFile;
	}
	
	/**
	 * Returns the name of the game location
	 * @return name of the game location
	 */
	public String getName()
	{
		return this.name;
	}
	
	/**
	 * Returns the beanshell name for this game location
	 * @return beanshell name for this game location
	 */
	public String getBshName()
	{
		return this.bshName;
	}
	
	/**
	 * Returns the scaling factor for the game location's 
	 * characters
	 * @return scaling factor for the characters
	 */
	public float getScaling()
	{
		return this.scaleFactor;
	}
	
	/**
	 * Sets the background of the game location to the specified background
	 * @param pathBGImg path+filename of the new background image
	 */
	public void setBackground(String pathBGImg)
	{
		this.background = new ImageBackground(this.bsLoader.getImage(pathBGImg));
		for(AbstractCharacterSprite c : this.characters) c.setBackground(this.background);
		for(Item i : this.items) i.setBackground(this.background);
	}
	
	/**
	 * Sets an intermission screen that can be rendered instead
	 * of the usual game location content
	 * @param imgPath path+filename to the intermission screen image
	 */
	public void setIntermissionScreenEnabled(String imgPath)
	{
		this.intermissionScreen = bsLoader.getImage(imgPath);
	}
	
	/**
	 * Disables the intermission screen. After calling this
	 * method no intermission screen will be rendered until
	 * newly set.
	 */
	public void setIntermissionScreenDiabled()
	{
		this.intermissionScreen = null;
	}
	
	/**
	 * Disbales/pauses the rendering process
	 *
	 */
	public void disableRendering()
	{
		this.stopRendering = true;
	}
	
	/**
	 * Enables the rendering process after is has been stopped
	 *
	 */
	public void enableRendering()
	{
		this.stopRendering = false;
	}
	
	/**
	 * Determines whether the rendering process has been stopped
	 * @return is rendering stopped? --> true=yes, false=no
	 */
	public boolean isStoppedRendering()
	{
		return this.stopRendering;
	}

	/**
	 * Initializes the search grid for pathfinding.
	 * Call this method after all items and interactive hotspots have
	 * been added to the game location object.
	 *
	 */
	public void initSearchGrid()
	{	
		this.pathSearchGrid = new SearchGrid(this.imageWalkable,this.items,this.interactiveHotspots,this.exits,16);
	}
	
	/**
	 * Returns the path search grid with the current state of
	 * the grid
	 * @return path search grid
	 */
	public SearchGrid getPathSearchGrid()
	{
		return this.pathSearchGrid;
	}
	
	/**
	 * Starts to play the assigned background music for this game location
	 * @param loop loop the music? --> true=yes, false=no
	 */
	public void playMusic(boolean loop)
	{
		if(loop == true) this.bsMusic.play(this.bgMusicFile,BaseAudio.SINGLE_REPLAY);
		else this.bsMusic.play(this.bgMusicFile,BaseAudio.SINGLE);		
	}
	
	/**
	 * Starts to play the specified music file
	 * @param musicPath path+filename to music file to play
	 * @param loop loop the music? --> true=yes, false=no
	 */
	public void playMusic(String musicPath, boolean loop)
	{
		if(loop == true) this.bsMusic.play(musicPath,BaseAudio.SINGLE_REPLAY);
		else this.bsMusic.play(musicPath,BaseAudio.SINGLE);
	}
	
	/**
	 * Plays a sampled sound file once
	 * @param filePath path+filename of the sound file to play
	 */
	public void playSound(String filePath)
	{
		this.bsSound.play(filePath);
	}
	
	/**
	 * Returns the object responsible for playing sounds of 
	 * the game location
	 * @return object responsible for playing sounds
	 */
	public BaseAudio getSoundRenderer()
	{
		return this.bsSound;
	}
	
	/**
	 * Returns the currently playing background music.
	 * This can be different from the initially assigned
	 * background music file.
	 * @return currently playing background music (path+filename)
	 */
	public String getPlayingMusic()
	{
		return bsMusic.getLastAudioFile();
	}
	
	/**
	 * Adds an item to the game location
	 * @param item item to add to the game location
	 */
	public void addItem(Item item)
	{
		this.items.add(item);
		item.setBackground(this.background);
	}
	
	/**
	 * Removes an item from the game location, for example if the player picked
	 * it up
	 * @param item the item to remove
	 */
	public void removeItem(Item item)
	{
		this.removeGameObjects.add(item);
	}
	
	/**
	 * Return a list with all the items of the game location
	 * @return list with items of the location
	 */
	public ArrayList<Item> getItems()
	{
		return this.items;
	}
	
	/**
	 * Return a list with all the props of the game location
	 * @return list with props of the location
	 */
	public ArrayList<Prop> getProps()
	{
		return this.props;
	}
	
	/**
	 * Adds a prop to the list of props of this game location
	 * @param prop prop to add to the game location
	 */
	public void addProp(Prop prop)
	{
		this.props.add(prop);
	}
	
	/**
	 * Adds a character to the location. Only add non player characters. Use 
	 * addPlayer to add the player character to the game location
	 * @param character non player character to add to the location
	 */
	public void addCharacter(NPCSprite character)
	{
		this.characters.add(character);		
		character.setGameLocation(this);		
	}
	
	/**
	 * Removes a non player character from the game location
	 * @param character the character to remove
	 */
	public void removeCharacter(NPCSprite character)
	{
		this.removeGameObjects.add(character);
	}
	
	/**
	 * Adds an interactive hotspot to the game location
	 * @param hs interactive hotspot to add to the game location
	 */
	public void addInteractiveHotspot(InteractiveHotspot hs)
	{
		this.interactiveHotspots.add(hs);
	}
	
	/**
	 * Returns a list with the interactive hotspots of the game location
	 * @return interactive hotspots of the game location
	 */
	public ArrayList<InteractiveHotspot> getInteractiveHotspots()
	{
		return this.interactiveHotspots;
	}
	
	/**
	 * Checks the removeGameObject list for game objects that shall be
	 * threadsafely removed from the game loaction and removes them of course, if
	 * game objects were found in the list
	 *
	 */
	private void checkForGameObjectsToRemove()
	{
		if(this.removeGameObjects.isEmpty() == false)
		{
			for(AbstractGameObject remGO : this.removeGameObjects)
			{
				if(remGO instanceof NPCSprite)
				this.characters.remove(remGO);
				
				if(remGO instanceof Item)
					this.items.remove(remGO);
			}
		}
	}
	
	/**
	 * Checks for user triggered skipping of the currently spoken sentence
	 *
	 */
	private void checkForSkipSentence()
	{
		if(GameState.equals(GameState.DIALOG) 
				&& bsInput.isMousePressed(MouseEvent.BUTTON1))
		{			
			if(this.player.getGameActionScheduler().getGameActions().isEmpty() == false
					&& this.player.getGameActionScheduler().getFirst() instanceof Talk)
			{
				Talk talk = (Talk)this.player.getGameActionScheduler().getFirst();
				AbstractCharacterSprite talkingCharacter = 
					talk.getSentenceScheduler().getSentences().getFirst().getRelatedCharacter();
		
				bsSound.stop(talk.getSentenceScheduler().getSentences().getFirst().getAssignedSoundFile());
				talk.getSentenceScheduler().getSentences().removeFirst();
				talkingCharacter.getAnimScheduler().getQueue().removeFirst();
			}
		}
	}
	
	/**
	 * Returns a list containing the characters of the location. This
	 * list only contains non player characters 
	 * @return list with characters of the location
	 */
	public ArrayList<NPCSprite> getNonPlayerCharacters()
	{
		return this.characters;
	}
	
	/**
	 * Returns the player object in the game location
	 * @return player object
	 */
	public Player getPlayer()
	{
		return this.player;
	}
	
	/**
	 * Adds the player character to the game location
	 * @param player player object
	 */
	public synchronized void addPlayer(Player player)
	{
		this.player = player;
		this.player.setGameLocation(this);
	}
	
	/**
	 * Removes the player from this game location
	 *
	 */
	public synchronized void removePlayer()
	{
		this.player = null;
	}
	
	/**
	 * Adds an exit to the game location with the specified parameters
	 * @param name name of the exit
	 * @param bshName beanshell name for the location exit
	 * @param x x-coordinate
	 * @param y y-coordinate
	 * @param width width of the exit
	 * @param height height of the exit
	 * @param newX new x-coordinate for the player
	 * @param newY new y-coordinate for the player
	 * @param usable determines if the exit can be initially used
	 * @param destination the destination game location this exit leads to
	 */
	public void addExit(String name,String bshName, double x, double y, int width, int height, double newX, double newY, boolean usable, GameLocation destination)
	{
		this.exits.add(new LocationExit(name,bshName,x,y,width,height,newX,newY,usable,this,destination,this.bsInput));
	}
	
	/**
	 * Returns the list of exits of this game location
	 * @return list of exits of this game location
	 */
	public ArrayList<LocationExit> getExits()
	{
		return this.exits;
	}
	
	/**
	 * Determines wheter this is the location that is currently shown
	 * @return is location shown? --> true=yes, false=no
	 */
	public boolean isActive()
	{
		return this.isActive;
	}
	
	/**
	 * Enables or disables the game loaction. If it is set to active, it is the
	 * currently shown one. If it is set to not active, the game location is not the
	 * shown one anymore
	 * @param active activate location (true) or deactive location (false)
	 */
	public void setActive(boolean active)
	{
		this.isActive = active;
	}
	
	/**
	 * Sorts the list containing all game objects that ought to
	 * be drawn to the screen by its y value. The highest y value
	 * will be the game object draw at last and therefore be drawn
	 * above the others.
	 *
	 */
	private synchronized void sortRenderList()
	{
		// copy the game objects of the location to the
		// render list to be sorted
		this.sortedRenderList.clear();
		this.sortedRenderList.addAll(this.items);
		this.sortedRenderList.addAll(this.characters);
		this.sortedRenderList.addAll(this.props);
		if(this.player != null) this.sortedRenderList.add(this.player);
		
		Collections.sort(this.sortedRenderList);
	}
	
	/**
	 * Private helper method to set the correct status of the current game cursor
	 *
	 */
	private void updateCursors()
	{
		// update cursors in a certain hierarchy
		CursorManager.setCursor(Cursors.NORMAL); // reset the cursor
		if(this.player != null && this.pathSearchGrid != null) // neccessary to avoid null pointer exception while exiting location
		{
			if(this.pathSearchGrid.getGridNode(bsInput.getMouseX(),bsInput.getMouseY()).isWalkable() == false 
					&& this.player.isLookingAtInventory() == false)
			{
				CursorManager.setCursor(Cursors.NO_ACTION);
			}
			
			for(LocationExit ex : this.exits)
			{
				if(ex.isUsable() && ex.isMouseOverHotspot())
				{
					CursorManager.setCursor(Cursors.LEAVE);
				}				
			}
			for(Item it : this.items)
			{
				if(it.isPickable() && it.getHotspot().isMouseOverHotspot())
				{
					CursorManager.setCursor(Cursors.PICK);
				}				
			}
			for(AbstractCharacterSprite ch : this.characters)
			{
				if(ch instanceof Player == false)
				{
					if(this.player.getInventItem() == null && 
							ch.getHotspot().isMouseOverHotspot())
					{
						CursorManager.setCursor(Cursors.TALK);
					}
					if(this.player.getInventItem() != null &&
							ch.getHotspot().isMouseOverHotspot())
					{
						CursorManager.setCursor(Cursors.USE);
					}
				}			
			}
			for(InteractiveHotspot hs : this.interactiveHotspots)
			{
				if(this.player.getInventItem() != null &&
						hs.isMouseOverHotspot())
				{
					CursorManager.setCursor(Cursors.USE);
				}		
			}						
		}
	}
	
	/**
	 * Helper method to handle selecting an item from the inventory
	 *
	 */
	private void checkForInventoryItemSelection()
	{		
		if(this.player.getInventory().isLookingAtInventory() 
				&& bsInput.isMousePressed(MouseEvent.BUTTON1))
		{				
			for(Item selItem : this.player.getInventory().getInventoryItems())
			{					
				if(selItem.getInventHotspot().isMouseOverInventHotspot())
				this.player.getInventory().setSelectedInventoryItem(selItem);								
			}
		}
		
		// deselect inventory item with right click
		if(bsInput.isMousePressed(MouseEvent.BUTTON3)) 
			this.player.getInventory().setSelectedInventoryItem(null);
	}
	
	/**
	 * Helper method to handle picking of items
	 *
	 */
	private void checkForPickAction()
	{
		for (Item i : this.items)
		{
			// determine, if the item gets picked
			if(i.getHotspot().isMouseOverHotspot())
			{
				if(i.isPickable() && bsInput.isMousePressed(MouseEvent.BUTTON1))
				i.setNotedForPicking(true);
			}
		}	
		
		Item pickedItem = null; // item that gets possibly picked
		// pick an item that was noted for picking, if the player is near to it
		for(Item i : this.items)
		{				
			if(this.player.hasArrivedAtTarget() && i.isNotedForPicking())// && i.isInPickingRange(this.player))					
			{
				pickedItem = i;
			}	
		}	
		if(pickedItem != null)
		{
			BeanShell.invoke("pick_"+pickedItem.getBshName()+"()");
		}
	}
	
	/**
	 * Helper method to handle picking of items
	 *
	 */
	private void checkForUseAction()
	{
		if(bsInput.isMousePressed(MouseEvent.BUTTON1)
				&& GameState.getState() == GameState.NORMAL
				&& this.player.getInventory().isLookingAtInventory() == false
				&& this.player.getInventItem() != null)					
		{		
			InteractiveHotspot ihs = this.getInteractiveHotspotUnderMouse();
			AbstractGameObject npc = this.getGameObjectUnderMouse();
			
			if(ihs != null)
			{
				BeanShell.invoke("use_"+ihs.getBshName()+"()");
				this.player.getInventory().setSelectedInventoryItem(null);
				GameState.setState(GameState.NO_INPUT_PROCESSING);
			}
			if(npc != null && npc instanceof NPCSprite)
			{				
				BeanShell.invoke("use_"+npc.getBshName()+"()");
				this.player.getInventory().setSelectedInventoryItem(null);
				GameState.setState(GameState.NO_INPUT_PROCESSING);
			}	
		}
	}
	
	/**
	 * Helper method to handle the action, that if the user 
	 * right clicks on an game object the player character will
	 * tell its description. 
	 *
	 */
	private void checkForDescribeAction()
	{
		// check if the user right clicked on a game object
		// if so, then let the player character tell the description
		// of the game object
		if(bsInput.isMousePressed(MouseEvent.BUTTON3))
		{			
			for(Item item : this.items)
			{
				if(item.isVisible() == true &&
						item.getHotspot().isMouseOverHotspot()
						&& this.player.getGameActionScheduler().
						containsActionsOtherThanWalk() == false)
				{															
					BeanShell.invoke("describe_"+item.getBshName()+"()");
				}
			}
			
			for(NPCSprite character : this.characters)
			{
				if(character.getHotspot().isMouseOverHotspot()
						&& this.player.getGameActionScheduler().
						containsActionsOtherThanWalk() == false)
				{				
					BeanShell.invoke("describe_"+character.getBshName()+"()");					
				}
			}
			
			for(InteractiveHotspot hs : this.interactiveHotspots)
			{
				if(hs.isMouseOverHotspot() && hs.isActive()
						&& this.player.getGameActionScheduler().
						containsActionsOtherThanWalk() == false)
				{				
					BeanShell.invoke("describe_"+hs.getBshName()+"()");
				}
			}
		}
	}
	
	/**
	 * Helper method that adds a walk action to the game action
	 * scheduler if possible
	 *
	 */
	private void checkForWalkAction()
	{
		if((bsInput.isMousePressed(MouseEvent.BUTTON1)
				|| bsInput.isMousePressed(MouseEvent.BUTTON3))
				&& GameState.getState() == GameState.NORMAL
				&& this.player.getInventory().isLookingAtInventory() == false)					
		{		
			
			// if there is no action in the scheduler a walk action can
			// be added
			if(this.player.getGameActionScheduler().getGameActions().isEmpty())
			{				
				AbstractGameObject go = this.getGameObjectUnderMouse();
				InteractiveHotspot ihs = this.getInteractiveHotspotUnderMouse();
				LocationExit exit = this.getExitUnderMouse();
				
				if(go != null)
				{
					if(this.player.getX() < go.getX())
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.max(go.getX()-player.getWidth(),0),go.getY(),bsInput);
					}
					else
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.min(go.getX()+go.getWidth(),this.background.getWidth()-1),go.getY(),bsInput);
					}
				}
				else if(ihs != null && (exit == null || exit.isUsable() == false))
				{					
					if(this.player.getX() < ihs.getX())
					{									
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.max(ihs.getX()-player.getWidth(),0),ihs.getY(),bsInput);
					}
					else
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.min(ihs.getX()+ihs.getWidth(),this.background.getWidth()-1),ihs.getY(),bsInput);
					}
				}
				else if(exit != null && exit.isUsable() == true)
				{
					PathFinder.addPath(player, this.pathSearchGrid,
							exit.getX()+exit.getWidth()/2,exit.getY()+exit.getHeight(),bsInput);				
				}
				else
				{
					PathFinder.addPath(player, this.pathSearchGrid,
							bsInput.getMouseX(),bsInput.getMouseY(),bsInput);
				}
				
			}
			else if(this.player.getGameActionScheduler().containsActionsOtherThanWalk() == false)
			{
				//this.player.getGameActionScheduler().clear();
				//this.player.getAnimScheduler().clear();
				
				AbstractGameObject go = this.getGameObjectUnderMouse();
				InteractiveHotspot ihs = this.getInteractiveHotspotUnderMouse();
				LocationExit exit = this.getExitUnderMouse();
				
				if(go != null)
				{
					if(this.player.getX() < go.getX())
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.max(go.getX()-player.getWidth(),0),go.getY(),bsInput);
					}
					else
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.min(go.getX()+go.getWidth(),this.background.getWidth()-1),go.getY(),bsInput);
					}
				}
				else if(ihs != null && (exit == null || exit.isUsable() == false))
				{
					if(this.player.getX() < ihs.getX())
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.max(ihs.getX()-player.getWidth(),0),ihs.getY(),bsInput);
					}
					else
					{
						PathFinder.addPath(player, this.pathSearchGrid,
								Math.min(ihs.getX()+ihs.getWidth(),this.background.getWidth()-1),ihs.getY(),bsInput);
					}
				}
				else if(exit != null && exit.isUsable() == true)
				{
					PathFinder.addPath(player, this.pathSearchGrid,
							exit.getX()+exit.getWidth()/2,exit.getY()+exit.getHeight(),bsInput);				
				}
				else
				{
					PathFinder.addPath(player, this.pathSearchGrid,
							bsInput.getMouseX(),bsInput.getMouseY(),bsInput);
				}
			}
		}
	}

	/**
	 * Helper method to handle starting of a conversation
	 *
	 */
	private void checkForTalkAction()
	{
		if(this.player != null 
				&& this.player.getInventItem() == null 
				&& bsInput.isMousePressed(MouseEvent.BUTTON1))
		{						
			for(NPCSprite character : this.characters)
			{
				if(character.getHotspot().isMouseOverHotspot()
						&& this.player.getGameActionScheduler().
						containsActionsOtherThanWalk() == false)
				{
					// clears the input actions to avoid immediate 
					// selection of a question
					bsInput.refresh();
					if(!GameState.equals(GameState.DIALOG_SELECTION))
					{
						GameState.setState(GameState.DIALOG_SELECTION);
					}
					BeanShell.invoke("talk_"+character.getBshName()+"()");					
				}
			}
		}
	}
	
	/**
	 * Helper method to handle the action of useing an exit
	 *
	 */
	private void checkForLeaveAction()
	{
		// only check for a leave action if the player
		// is not trying to use an item
		if(this.player.getInventItem() == null)
		{
			// check if an exit was taken
			for(LocationExit ex : this.exits)
			{
				// set the exit to be noted for use, if the player clicks on it
				if(ex.isUsable() && ex.isMouseOverHotspot() 
						&& bsInput.isMousePressed(MouseEvent.BUTTON1))
				{
					ex.setNotedForUse(true);
				}
				
				if(//ex.isInRange(this.player) && 
						ex.isNotedForUse())
				{
					// only add a leave action if the player needs to walk
					// to the exit to use
					/*
					if(player.getGameActionScheduler().getGameActions().isEmpty() == false
							&& player.getGameActionScheduler().getGameActions()
							.getLast() instanceof Walk)
					*/
					this.player.addAction(new Leave(player,ex));
				}
			}
		}
	}
	
	/**
	 * Helper method to perform the update operation
	 * for the player object
	 *
	 */
	private void updatePlayer(long elapsedTime)
	{
		// update player
		if(this.player != null)this.player.update(elapsedTime);
	}
	
	/**
	 * Helper method to perform the update operation
	 * for the non player characters
	 * @param elapsedTime time between two frames
	 */
	private void updateNonPlayerCharacters(long elapsedTime)
	{
		// update characters
		for (NPCSprite c : this.characters)
		{
			c.update(elapsedTime);
		}
	}
	
	/**
	 * Helper method to perform the update operation
	 * for the non player characters
	 * @param elapsedTime time between two frames
	 */
	private void updateItems(long elapsedTime)
	{
		for(Item i : this.items) i.update(elapsedTime);			
	}
	
	/*
	private void updateInteractiveHotspots(long elapsedTime)
	{
		for(InteractiveHotspot hs : this.interactiveHotspots)
		{
			hs.update(hs.getX(),hs.getY(),hs.getWidth(),hs.getHeight());
		}
	}
	*/
	
	/**
	 * Updates the content of the game location
	 * @param elapsedTime time between two frames
	 */
	public void update(long elapsedTime)
	{					
		// only update anything of this location, if it is actually active
		if(this.isActive == true)
		{	
			this.sortRenderList();
			//System.out.println("GL Name: "+this.getBshName());
			//for(AbstractCharacterSprite c : this.characters)System.out.println("Char name: "+c.getBshName());
			if(GameState.equals(GameState.NORMAL) 
					&& this.player != null)
			{
				this.checkForWalkAction();
				this.checkForTalkAction();
				this.checkForLeaveAction();
				this.checkForUseAction();
				this.checkForPickAction();
				this.checkForInventoryItemSelection();
				this.checkForDescribeAction();			
				this.checkForGameObjectsToRemove();				
							
				
				this.background.update(elapsedTime);			
				this.updatePlayer(elapsedTime);
				this.updateNonPlayerCharacters(elapsedTime);
				this.updateItems(elapsedTime);
				//this.updateInteractiveHotspots(elapsedTime);
				this.updateCursors();	
				
				// for path debugging
				if(bsInput.isKeyPressed(KeyEvent.VK_P)) this.visGrid = !this.visGrid;
				if(this.visGrid == true && bsInput.isMousePressed(MouseEvent.BUTTON2))
				{					
					AStarSearch search = new AStarSearch();
					this.pathNodes.clear();
					this.pathNodes = search.findPath(
							this.pathSearchGrid.getGridNode(player.getX(), player.getY()),
							this.pathSearchGrid.getGridNode(bsInput.getMouseX(), bsInput.getMouseY()));			
				}				
			}
			
			if(GameState.equals(GameState.DIALOG_SELECTION)
					|| GameState.equals(GameState.DIALOG))
			{
				this.background.update(elapsedTime);			
				this.updatePlayer(elapsedTime);
				this.updateNonPlayerCharacters(elapsedTime);
				this.updateItems(elapsedTime);				
				CursorManager.setCursor(Cursors.NORMAL);
				this.checkForSkipSentence();
				if(GameState.equals(GameState.DIALOG))
					CursorManager.setCursor(Cursors.HIDE);
			}
			
			if(GameState.equals(GameState.NO_INPUT_PROCESSING))
			{
				this.background.update(elapsedTime);			
				this.updatePlayer(elapsedTime);
				this.updateNonPlayerCharacters(elapsedTime);
				this.updateItems(elapsedTime);				
				CursorManager.setCursor(Cursors.HIDE);				
			}
			
			if(GameState.equals(GameState.TRANSITION_TO_CUTSCENE) 
					|| GameState.equals(GameState.TRANSITION_FROM_CUTSCENE)
					|| GameState.equals(GameState.CUTSCENE))
			{
				this.background.update(elapsedTime);			
				this.updatePlayer(elapsedTime);
				this.updateNonPlayerCharacters(elapsedTime);
				this.updateItems(elapsedTime);				
				CursorManager.setCursor(Cursors.HIDE);				
			}
			
			if(GameState.equals(GameState.TRANSITION_FROM_CUTSCENE))
			{
				this.counter++;
				if(this.counter > 1)
				{
					this.counter = 0;
					GameState.setState(GameState.NORMAL);
				}
				
			}
			
			if(this.pathSearchGrid == null)
			{				
				this.initSearchGrid();
			}
		}				
	}
	

	
	/**
	 * Renders the content of the game loaction
	 * @param g
	 */
	public void render(Graphics2D g)
	{
		/*
		if(GameState.equals(GameState.TRANSITION_TO_CUTSCENE)
				|| GameState.equals(GameState.TRANSITION_FROM_CUTSCENE))
		{
			g.setColor(Color.BLACK);
			g.fillRect(0, 0, 1024, 768);
			this.counter++;
			return;
		}
		*/
		
		// only render anything of this game location if it is actually active
		if(this.isActive == true && this.stopRendering == false)
		{					
			if(this.intermissionScreen != null)
			{
				// if there is an intermission screen to draw
				// render it and return immediately in order
				// not to render anything else
				g.drawImage(this.intermissionScreen, null, 0, 0);
				return;
			}
			
			this.background.render(g);
			
			for(InteractiveHotspot hs : this.interactiveHotspots) hs.render(g);
			
			// draw game objects
			for(AbstractGameObject sortedGO : this.sortedRenderList)
			{
				if(sortedGO instanceof Item)
				{
					Item item = (Item)sortedGO;
					if(item.isVisible()) sortedGO.render(g);
					
				}
				else sortedGO.render(g);
			}
			
			// draw scene hotspot things, like the name after
			// the game objects have been drawn, to draw the
			// names always in front of the object itself
			for(AbstractGameObject sortedGO : this.sortedRenderList)
			{
				// do not show player name
				if(sortedGO instanceof Player == false)
				sortedGO.getHotspot().render(g);
			}
			
			if(this.player != null &&
					this.player.getInventItem() != null)
				g.drawImage(player.getInventItem().getInventoryImage(),
						bsInput.getMouseX(), bsInput.getMouseY()+32,
						player.getInventItem().getInventoryImage().getWidth(),
						player.getInventItem().getInventoryImage().getHeight(),
						null);
			
			if(this.visGrid == true) this.visualizeGrid(g);			
		}
	}
	
	/**
	 * Helper method to render the intermission screen
	 * @param g
	 */
	private void renderIntermissionScreen(Graphics2D g)
	{
		g.drawImage(this.intermissionScreen, null, 0, 0);
	}
	
	/**
	 * Debug method that visualizes the search grid.
	 * @param g
	 */
	private void visualizeGrid(Graphics2D g)
	{
		// render search grid
		GridNode[][] gn = this.pathSearchGrid.getGridNodes();
		for(int y=0; y<gn.length; y++)
		{
			for(int x=0; x<gn[y].length; x++)
			{
				// draw grid
				g.setColor(Color.BLACK);
				g.drawRect(
						(int)gn[y][x].getX(),(int)gn[y][x].getY(),
						gn[y][x].getWidth(),gn[y][x].getHeight());										
			}
		}
		
		// show calculated path		
		
		for(GridNode pn : pathNodes)
		{		
			g.setColor(Color.MAGENTA);
			g.drawRect((int)pn.getX(), (int)pn.getY(),
					pn.getWidth(), pn.getHeight());
		}
		
		// show neighbour grid nodes
		
		if(bsInput.isMousePressed(MouseEvent.BUTTON2))
		{
			GridNode selGN = this.pathSearchGrid.getGridNode(player.getMidX(),player.getY());
			for(GridNode neigh : selGN.getNeighbors())
			{		
				g.setColor(Color.BLUE);
				g.drawRect((int)neigh.getX(), (int)neigh.getY(),
						neigh.getWidth(), neigh.getHeight());
			}
		}		
	}
	
	/**
	 * Returns the game object that is currently located under the mouse
	 * cursor. The method return null if there is no such game object
	 * @return game object under the mouse cursor
	 */
	private AbstractGameObject getGameObjectUnderMouse()
	{
		for(AbstractCharacterSprite c: this.characters)
		{
			if(c.getHotspot().isPointInHotspot(bsInput.getMouseX(), bsInput.getMouseY()))
			{
				return c;
			}
		}
		
		for(Item i : this.items)
		{
			if(i.getHotspot().isPointInHotspot(bsInput.getMouseX(), bsInput.getMouseY()))
			{				
				return i;
			}
		}
		
		return null;
	}
	
	/**
	 * Returns the interactive hotspot that is currently located under the mouse
	 * cursor. The method return null if there is no such interactive
	 * hotspot
	 * @return interactive hotspot under the mouse cursor
	 */
	private InteractiveHotspot getInteractiveHotspotUnderMouse()
	{
		for(InteractiveHotspot hs : this.interactiveHotspots)
		{
			if(hs.isPointInHotspot(bsInput.getMouseX(), bsInput.getMouseY()))
			{				
				return hs;
			}
		}
		
		return null;
	}
	
	/**
	 * Returns the location exit that is currently located under the mouse
	 * cursor. The method return null if there is no such exit
	 * @return location exit under the mouse cursor
	 */
	private LocationExit getExitUnderMouse()
	{
		for(LocationExit exit : this.exits)
		{
			if(exit.isPointInHotspot(bsInput.getMouseX(), bsInput.getMouseY()))
			{				
				return exit;
			}
		}
		return null;
	}
}
