package main;

import com.golden.gamedev.*;
import com.golden.gamedev.engine.audio.JavaLayerMp3Renderer;

import cursor.CursorManager;
import cursor.Cursors;
import cursor.GameCursor;
import java.awt.*;
import java.awt.event.*;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

import scripting.BeanShell;
import state.GameState;
import util.FileFinder;
import xml.LocationLoader;

import location.GameLocation;
import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.NPCSprite;
import gameobjects.characters.Player;
import gameobjects.item.Item;

/**
 * This is the main class and starting point of the adventure game
 * 
 * @author Frank Bruns
 */
public class AdventureMain extends Game
{
	{
		this.distribute = false;
	}

	private ArrayList<GameLocation> locations = null;

	private BeanShell bsh = null;
	
	private boolean renderCoords = false;
	
	private boolean firstLoop = true;

	public void initResources()
	{				
		this.locations = new ArrayList<GameLocation>();
		this.initCursors();
		this.bsh = new BeanShell();
		
		FileFinder ff = new FileFinder("./location_data/",".xml");
		LocationLoader locLoader = new LocationLoader(bsLoader, bsInput,
				bsSound, bsMusic);
		this.locations = locLoader.getLocationFromXML(ff.getFoundFiles());
		this.locations = locLoader.getLocationsWithAddedExits(locations);
		this.locations = locLoader.getLocationsWithAddedPlayer(locations);
		
		
		// set the location to active that contains the player
		// object
		for(GameLocation loc : this.locations)
		{
			if(loc.getPlayer() != null)
			{
				loc.setActive(true);
				loc.playMusic(true);
			}
		}

		// setup beanshell interpreter on gamelocations
		for (GameLocation l : this.locations)
			this.bsh.addGameLocation(l);
		this.bsh.initGameLocations();
	}

	@Override
	protected void initEngine()
	{
		super.initEngine();		
		bsSound.setBaseRenderer(new JavaLayerMp3Renderer());
	}
	
	@Override
	public void render(Graphics2D g)
	{				
		for (GameLocation l : this.locations)
			l.render(g);
		CursorManager.render(g, this.bsInput);

		this.renderMouseCoords(g);
	}

	@Override
	public void update(long elapsedTime)
	{		
		// run the intro in the second loop of the update() method 
		if(this.firstLoop == false 
				&& GameState.equals(GameState.INTRO_MODE))
		{
			GameState.setState(GameState.NORMAL);
			BeanShell.executeScript("intro/intro.bsh");
		}
		
		for (GameLocation l : this.locations)
			l.update(elapsedTime);
		this.handleGeneralKeyEvents();		
		
		if(this.firstLoop == true)
		{
			GameState.setState(GameState.INTRO_MODE);
			this.firstLoop = false;
		}
	}

	/**
	 * Helper method that draws the current mouse coordinates to screen
	 * @param g
	 */
	private void renderMouseCoords(Graphics2D g)
	{
		if(this.renderCoords == true)
		{
			int x = this.bsInput.getMouseX();
			int y = this.getMouseY();
	
			g.setColor(Color.BLACK);
			g.fillRect(10, 100, 100, 25);
			g.setColor(Color.WHITE);
			g.drawString(new String("x:" + x + "  y:" + y), 20, 120);
		}
	}

	private void handleGeneralKeyEvents()
	{
		if (bsInput.getKeyPressed() == KeyEvent.VK_ESCAPE)
		{
			System.exit(0);
		}
		if (bsInput.getKeyPressed() == KeyEvent.VK_C)
		{
			this.renderCoords = !this.renderCoords;
		}
		if(bsInput.getKeyPressed() == KeyEvent.VK_S)
		{
			try
			{
				FileOutputStream fos = new FileOutputStream("saves/test.sav");
				ObjectOutputStream oos = new ObjectOutputStream(fos);
				
				for(GameLocation loc : this.locations)
				{
					System.out.println("[AdventureMain] Saving location: "+loc.getName());
					
					for(NPCSprite c : loc.getNonPlayerCharacters())
					{
						oos.writeObject(c.getName());
					}
					for(Item i : loc.getItems())
					{
						oos.writeObject(i.getName());
					}
					
				}
			}
			catch (FileNotFoundException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * Helper method to initialize the game cursors
	 * 
	 */
	private void initCursors()
	{
		CursorManager.addCursor(new GameCursor(Cursors.NORMAL, bsLoader
				.getImage("./images/cursors/cursor_normal.png")));

		CursorManager.addCursor(new GameCursor(Cursors.TALK, bsLoader
				.getImage("./images/cursors/cursor_talk.png")));

		CursorManager.addCursor(new GameCursor(Cursors.PICK, bsLoader
				.getImage("./images/cursors/cursor_pick.png")));

		CursorManager.addCursor(new GameCursor(Cursors.LEAVE, bsLoader
				.getImage("./images/cursors/cursor_leave.png")));

		CursorManager.addCursor(new GameCursor(Cursors.USE, bsLoader
				.getImage("./images/cursors/cursor_use.png")));

		CursorManager.addCursor(new GameCursor(Cursors.NO_ACTION, bsLoader
				.getImage("./images/cursors/cursor_no_action.png")));
		
		CursorManager.addCursor(new GameCursor(Cursors.HIDE, bsLoader
				.getImage("./images/cursors/cursor_invis.png")));

		CursorManager.setCursor(Cursors.NORMAL);
		this.bsInput.setMouseVisible(false);
	}

	/**
	 * @param args Command line parameter
	 */
	public static void main(String[] args)
	{
		AdventureMain game = new AdventureMain();
		GameLoader gameLoader = new GameLoader();
		gameLoader.setup(game, new Dimension(1024, 768), new Boolean(args[0]));
		gameLoader.start();
	}
}
