package path;

import gameaction.AbstractGameAction;
import gameaction.Walk;
import gameobjects.characters.AbstractCharacterSprite;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.LinkedList;
import java.util.List;

import com.golden.gamedev.engine.BaseInput;

/**
 * A class that provides static methods to return a path from the characters start
 * location to his target location by constructing a list of Walk game actions,
 * @author Frank Bruns
 *
 */
public class PathFinder
{
	private static List<AbstractGameAction> pathWalkActions = new LinkedList<AbstractGameAction>();
	private static AStarSearch search = new AStarSearch();
	
	/**
	 * Returns a list of Walk game actions that represents a path from the characters
	 * current position to its destionation coordinates
	 * @param c character to find the path for
	 * @param imgWalkableArea image that contains the walkable area
	 * @param x destination x coordinate
	 * @param y destination y coordinate
	 * @return list of Walk events
	 * @deprecated do nut use this method any more
	 */
	public static List<AbstractGameAction> getPath(AbstractCharacterSprite c, BufferedImage imgWalkableArea, double x, double y)
	{
		pathWalkActions.clear();

		if(imgWalkableArea.getRGB((int)x,(int)y) != Color.WHITE.getRGB())
		{
			pathWalkActions.add(new Walk(c,(int)x,(int)y));
		}				
		
		return pathWalkActions;
	}
	
	
	/**
	 * Returns a list of Walk game actions that represents a path from the characters
	 * current position to its destionation coordinates
	 * @param c character to find the path for
	 * @param grid the search grid to calculate the path in
	 * @param x destination x coordinate
	 * @param y destination y coordinate
	 * @return list of Walk events
	 */
	public static List<AbstractGameAction> getPath(AbstractCharacterSprite c, SearchGrid grid, double x, double y, BaseInput bsInput)
	{		
		if(grid.getGridNode(bsInput.getMouseX(),bsInput.getMouseY()).isWalkable() == true
				|| grid.getGridNode(x,y).isWalkable() == true)
		{			
			pathWalkActions.clear();					
			List<GridNode> pathNodes = search.findPath(
					grid.getGridNode(c.getMidX(), c.getY()),
					grid.getGridNode(x, y));
			
			for(GridNode pn : pathNodes)
			{			
				int nodeCenterX = (int)pn.getX()+grid.getGridNodeSize()/2;
				int nodeCenterY = (int)pn.getY()+grid.getGridNodeSize()/2;
				
				// stop adding walk actions, if the path would lead over
				// unwalkable areas
				if(grid.getImageWalkable().getRGB(nodeCenterX,nodeCenterY) == Color.WHITE.getRGB())
				{
					break;
				}
				else pathWalkActions.add(new Walk(c,(int)pn.getX(),(int)pn.getY()));
			}			
		}
		return pathWalkActions;
	}
	
	/**
	 * Finds a list of Walk game actions that represents a path from the characters
	 * current position to its destionation coordinates and adds it to
	 * the specified characters game action queue
	 * @param c character to find the path for
	 * @param grid the search grid to calculate the path in
	 * @param x destination x coordinate
	 * @param y destination y coordinate
	 * @param bsInput
	 */
	public static void addPath(AbstractCharacterSprite c, SearchGrid grid, double x, double y, BaseInput bsInput)
	{		
		if(grid.getGridNode(bsInput.getMouseX(),bsInput.getMouseY()).isWalkable() == true
				|| grid.getGridNode(x,y).isWalkable() == true)
		{
			c.getGameActionScheduler().clear();
			//c.getAnimScheduler().clear();
			
			List<GridNode> pathNodes = search.findPath(
					grid.getGridNode(c.getX(), c.getY()),
					grid.getGridNode(x, y));
			
			Path path = new Path();			
			for(GridNode pn : pathNodes)
			{			
				int nodeCenterX = (int)pn.getX()+grid.getGridNodeSize()/2;
				int nodeCenterY = (int)pn.getY()+grid.getGridNodeSize()/2;
				
				// stop adding points to the path, if the path would lead over
				// unwalkable areas
				if(grid.getImageWalkable().getRGB(nodeCenterX,nodeCenterY) == Color.WHITE.getRGB())
				{
					break;
				}
				else path.addPoint(pn.getX(), pn.getY());
			}
			
			c.addAction(new Walk(c,path));
		}
	}
}
