package scripting;

import gameobjects.characters.AbstractCharacterSprite;
import gameobjects.characters.Player;
import gameobjects.item.Item;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import util.FileFinder;
import location.GameLocation;
import location.InteractiveHotspot;
import location.LocationExit;
import bsh.EvalError;
import bsh.Interpreter;

/**
 * This class provides an interface to use beanshell scripts in the game
 * @author Frank Bruns
 *
 */
public class BeanShell
{
	private static Interpreter bsh = null;
	private ArrayList<GameLocation> gameLocations = null;
	
	/**
	 * Creates a new beanshell interpreter object
	 */
	public BeanShell()
	{
		this.bsh = new Interpreter();
		this.gameLocations = new ArrayList<GameLocation>();
		
		// bind the common methods to the interpreter
		try
		{
			bsh.source("./location_data/common.bsh");
		}
		catch (Exception e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * Adds a game location to the beanshell script interpreter
	 * @param g game location to add to the beanshell interpreter
	 */
	public synchronized void addGameLocation(GameLocation g)
	{
		this.gameLocations.add(g);
	}
	
	/**
	 * Intialize the beanshell interpreter with the game locations that have
	 * been added to the interpreter before
	 *
	 */
	public void initGameLocations()
	{		
		for(GameLocation gameLoc : this.gameLocations)
		{
			try
			{				
				// provide .bsh script file
				bsh.source(gameLoc.getScriptFileName());			
				
				// bind game location to bsh interpreter
				bsh.set(gameLoc.getBshName(), gameLoc);			
				
				// bind player object and all its dialogs 
				// to the bsh interpreter
				if(gameLoc.getPlayer() != null)
				{
					Player player = gameLoc.getPlayer();
					bsh.set(player.getBshName(),player);										
				}
				
				// bind exits of game location to interpreter
				for(LocationExit exit : gameLoc.getExits())
				{
					bsh.set(exit.getBshName(), exit);
				}
				
				// bind non player characters of the game location to bsh interpreter
				for(AbstractCharacterSprite character : gameLoc.getNonPlayerCharacters())
				{				
					bsh.set(character.getBshName(),character);			
				}
				
				// bind items of the game location to bsh interpreter
				for(Item item : gameLoc.getItems())
				{
					bsh.set(item.getBshName(),item);
				}
				
				// bind interactive hotspots to bsh interpreter
				for(InteractiveHotspot hs : gameLoc.getInteractiveHotspots())
				{
					bsh.set(hs.getBshName(),hs);
				}																
			}
			catch(IOException exc)
			{
				System.out.println("Script IO exception at"+gameLoc.getName()+".bsh");
				exc.printStackTrace();
			}
			catch(EvalError exc)
			{
				System.out.println("Script evaluation error at"+gameLoc.getName()+".bsh");
				exc.printStackTrace();
			}
		}
		
		
		try{
			// bind player dialogs to the player by calling
			// the dialog beanshell files
			FileFinder ff = new FileFinder("./location_data/dialogs/",".bsh");				
			for(File file : ff.getFoundFiles())
			{
				bsh.source(file.getPath());
			}
			
			// bind the cutscenen to the player
			/*
			ff = new FileFinder("./location_data/cutscenes/",".bsh");				
			for(File file : ff.getFoundFiles())
			{
				bsh.source(file.getPath());
			}
			*/
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Invoke a method from the beanshell script of the current
	 * game location
	 * @param methodName method name in the .bsh script file
	 */
	public static void invoke(String methodName)
	{
		try
		{
			bsh.eval(methodName);
		}
		catch(EvalError exc)
		{
			System.out.println("Error while running scripted method '"
					+methodName+"'");
			exc.printStackTrace();
		}
	}
	
	/**
	 * Executes the specified beanshell script file line by line
	 * @param scriptFile bsh script file to execute
	 */
	public static void executeScript(String scriptFile)
	{
		try
		{
			bsh.source(scriptFile);
		}
		catch (FileNotFoundException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (EvalError e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
