package speech;

import gameaction.Talk;
import gameobjects.characters.Player;

import java.io.Serializable;
import java.util.LinkedList;

/**
 * This class represents a dialog between the player and another character.
 * It uses options for questions the player can ask
 * @author Frank Bruns
 *
 */
public class Dialog implements Serializable
{
	private LinkedList<DialogBranch> branches = null;
	private Player player = null;
	private String name = "no name yet";
	
	
	public Dialog(Player player, String name)
	{
		this.branches = new LinkedList<DialogBranch>();
		this.player = player;
		this.name = name;
	}
	
	/**
	 * Returns the name of the dialog. The name is the identifier
	 * of the dialog
	 * @return identifying name of the dialog
	 */
	public String getName()
	{
		return this.name;
	}
	
	/**
	 * Adds a dialog branch to the dialog. If a branch with the same
	 * id already exits the dialog branch won't be added
	 * @param id id for the dialog branch
	 * @param question question that is associated to the dialog branch
	 * @param talkAction talk action to fire when choosing the dialog branch
	 */
	public void addDialogBranch(int id, String question, Talk talkAction)
	{
		for(DialogBranch db : this.branches)
		{
			if(db.getID() == id) return; // leave the method
		}
		this.branches.add(new DialogBranch(id, question, talkAction));
		
		// resort the braches to put the aborter talk action 
		// at the last position
		if(talkAction.isAborter() == false)
		{
			DialogBranch aborterBranch = null;
			
			for(DialogBranch db : this.branches)
			{
				if(db.getTalkAction().isAborter() == true)
				{
					 aborterBranch = db;
					 break;
				}
			}
			
			if(aborterBranch != null)
			{
				this.branches.remove(aborterBranch);
				this.branches.addLast(aborterBranch);
			}
		}
	}
	
	/**
	 * Starts the dialog branch which is specified by the id 
	 * @param id id of the dialog branch to be run
	 */
	public void runDialogBranch(int id)
	{
		if(this.player.hasArrivedAtTarget())
		{
			DialogBranch branch = this.getBranch(id);
			this.player.getGameActionScheduler().add(branch.getTalkAction());
			if(branch.getTalkAction().isAborter() == false)
				branch.setExclude(true);
		}
	}
	
	/**
	 * Returns the dialog branch with the specified id, if it exists, else
	 * this method returns null
	 * @param id id of the dialog branch to be found
	 * @return dialog branch with the specified id
	 */
	public DialogBranch getBranch(int id)
	{
		for(DialogBranch branch : this.branches)
		{
			if(branch.getID() == id) return branch;
		}
		return null;
	}
	
	/**
	 * Returns a list of dialog branches of this dialog
	 * @return dialog branches of this dialog
	 */
	public LinkedList<DialogBranch> getBranches()
	{
		return this.branches;
	}
}
